/*******************************************************************************
 * Copyright (c) 2005 RadRails.org and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.radrails.org/legal/cpl-v10.html
 *******************************************************************************/

package org.radrails.db.core;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.sql.Connection;
import java.sql.SQLException;
import java.util.Collections;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import org.radrails.db.core.ibmdb2.IBMDB2DatabaseConnection;
import org.radrails.db.core.mysql.MySQLDatabaseConnection;
import org.radrails.db.core.oracle.OracleDatabaseConnection;
import org.radrails.db.core.postgresql.PostgreSQLDatabaseConnection;
import org.radrails.db.core.sqlite.SQLiteDatabaseConnection;
import org.radrails.db.core.sqlserver.SQLServerDatabaseConnection;

/**
 * Provides a helper class around the IDatabaseConnection class.
 * 
 * @author mkent
 * 
 * @version 0.2.1
 */
public class DatabaseHelper {

	private static DatabaseHelper instance;

	private Map connectionMap;

	/**
	 * Private constructor since this is a singleton. Use the getInstance()
	 * method to get an instance of this class.
	 */
	private DatabaseHelper() {
		connectionMap = new HashMap();

		// Initialize mappings
		connectionMap.put(IDatabaseConstants.ADAPTER_MYSQL,
				new MySQLDatabaseConnection());
		connectionMap.put(IDatabaseConstants.ADAPTER_POSTGRESQL,
				new PostgreSQLDatabaseConnection());
		connectionMap.put(IDatabaseConstants.ADAPTER_SQLITE,
				new SQLiteDatabaseConnection());
		connectionMap.put(IDatabaseConstants.ADAPTER_SQLITE3,
				new SQLiteDatabaseConnection());
		connectionMap.put(IDatabaseConstants.ADAPTER_OCI,
				new OracleDatabaseConnection());
		connectionMap.put(IDatabaseConstants.ADAPTER_SQLSERVER,
				new SQLServerDatabaseConnection());
		connectionMap.put(IDatabaseConstants.ADAPTER_DB2,
				new IBMDB2DatabaseConnection());
	}

	/**
	 * Gets an instance of this class.
	 * 
	 * @return A DatabaseHelper object.
	 */
	public static DatabaseHelper getInstance() {
		if (instance == null) {
			instance = new DatabaseHelper();
		}
		return instance;
	}

	/**
	 * Gets a list of the available adapters.
	 * 
	 * @return	A String array of the adapter names.
	 */
	public String[] getAdapters() {
		List adapterCol = new LinkedList(connectionMap.keySet());
		Collections.sort(adapterCol);
		return (String[]) adapterCol.toArray(new String[0]);
	}
	
	/**
	 * Tests is a database connection can be established based on the project
	 * and environment type.
	 * 
	 * @param project
	 *            The project to test a connection on.
	 * @param environment
	 *            The environment from the database.yml file to use. Use the
	 *            IDatabaseConstants for the available environment strings.
	 * 
	 * @return <code>true</code> if a database connection could be
	 *         established, <code>false</code> otherwise.
	 * @deprecated
	 */
	public boolean testConnection(File project, String environment) {
		boolean success = false;
		File databaseYML = new File(project + "/config/database.yml");
		try {
			FileReader reader = new FileReader(databaseYML);
			YMLFile parser = new YMLFile();
			parser.load(reader);
			DatabaseDescriptor database = null;
			if (environment.equals(IDatabaseConstants.ENV_TEST)) {
				database = parser.getDbTest();
			} else if (environment.equals(IDatabaseConstants.ENV_PRODUCTION)) {
				database = parser.getDbProduction();
			} else {
				// Default to development
				database = parser.getDbDevelopment();
			}
			DatabaseConnection connection = getConnectionForName(database
					.getAdapter());
			if (connection != null ) {
				success = connection.testConnect(database);
			}
		} catch (FileNotFoundException e) {
			DatabaseLog.logError("Database configuration file not found", e);
		} catch (IOException e) {
			DatabaseLog
					.logError("Error loading database configuration file", e);
		}
		return success;
	}

	/**
	 * Establishes a database connection based on the parameters.
	 * 
	 * @param	project	The location to the project.
	 * @param	environment	The environment to test
	 * 						(Use IDatabaseConstants).
	 * 
	 * @return	A java.sql.Connection object with a connection to the
	 * 			corresponding environment database.
	 * 
	 * @throws	SQLException	If an error occured while connecting to the
	 * 						database.
	 * @throws	InstantiationException	If the adaptor for the specified
	 * 									database could not be created.
	 * @throws	IllegalAccessException	If the user specified does
	 * 									not have access to the database.
	 * @throws	FileNotFoundException	If the project's database.yml file
	 * 								could not be found.
	 * @throws	IOException	If an I/O error occurs while connecting.
	 */
	public Connection connect(File project, String environment)
			throws SQLException, InstantiationException,
			IllegalAccessException, FileNotFoundException, IOException {
		File databaseYML = new File(project + "/config/database.yml");
		FileReader reader = new FileReader(databaseYML);
		DatabaseYml yml = new DatabaseYml();
		yml.load(reader);
		DatabaseDescriptor database = yml.getDescriptor(environment);
		
		DatabaseConnection connection = getConnectionForName(database
				.getAdapter());
		if(connection == null) {
			DatabaseLog.logError("No connection adapter found for: " + database.getAdapter(), null);
			throw new SQLException("No connection adapter found for: " + database.getAdapter());
		}
		Connection conn = connection.connect(database);

		return conn;
	}

	/**
	 * Gets the database connection based on the adapter name.
	 * 
	 * @param adapterName
	 *            The name of the adapter. Use the IDatabaseConstants class for
	 *            the available adapters.
	 * 
	 * @return The IDatabaseConnection object for the specified adapter, or
	 *         <code>null</code> if no IDatabaseConnection is defined for the
	 *         specified adapter name.
	 */
	public DatabaseConnection getConnectionForName(String adapterName) {
		return (DatabaseConnection) connectionMap.get(adapterName);
	}

} // DatabaseHelper
