/*******************************************************************************
 * Copyright (c) 2005 RadRails.org and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.radrails.org/legal/cpl-v10.html
 *******************************************************************************/

package org.radrails.db.core;

import java.io.FileNotFoundException;
import java.io.IOException;
import java.sql.Connection;
import java.sql.SQLException;
import java.util.Collections;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.IResourceChangeEvent;
import org.eclipse.core.resources.IResourceChangeListener;
import org.eclipse.core.resources.IResourceDelta;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.IPath;

/**
 * Represents a project's database connections.
 * 
 * @author mbaumbach
 * 
 * @version 0.3.0
 */
public class ProjectDatabaseManager implements IResourceChangeListener {

	private static final String DATABASE_YML_PATH = IPath.SEPARATOR + "config"
			+ IPath.SEPARATOR + "database.yml";

	private IProject project;

	private Map connections;

	private List pdmListeners;
	
	/**
	 * Creates a new ProjectDatabaseManager class based on a project.
	 * 
	 * @param project
	 *            The project associated with this ProjectDatabaseManager class.
	 */
	public ProjectDatabaseManager(IProject project) {
		this.project = project;

		// Create the connections map
		connections = Collections.synchronizedMap(new HashMap());
		
		// List for the listeners
		pdmListeners = new LinkedList();
		
		ResourcesPlugin.getWorkspace().addResourceChangeListener(this);
	}

	/**
	 * Attempts to get a connection for the specified environment.
	 * 
	 * @param env
	 *            The environment to get the connection for. Use
	 *            IDatabaseConstants.
	 * 
	 * @return A Connection to the database, or <code>null</code> if no
	 *         connection was established.
	 * 
	 * @throws IOException
	 *             If an I/O error occurs while getting the connection.
	 * @throws IllegalAccessException
	 *             If access to the database is denied.
	 * @throws InstantiationException
	 *             If the database cannot be created.
	 * @throws SQLException
	 *             If an error occurred while connecting to the database.
	 * @throws FileNotFoundException
	 *             If the database.yml file could not be found for the project.
	 */
	public Connection getConnection(String env) throws FileNotFoundException,
			SQLException, InstantiationException, IllegalAccessException,
			IOException {
		Connection conn = (Connection) connections.get(env);
		
		// If no connection was found or was closed attempt to connect
		if ( conn == null || conn.isClosed() ) {
			conn = DatabaseHelper.getInstance().connect(
				project.getLocation().toFile(), env);
			connections.put(env, conn);
		}

		return conn;
	}

	/**
	 * Adds a ProjectDatabaseManagerChangeListener to the list of
	 * listeners.
	 * 
	 * @param	listener	The listener to add.
	 * 
	 * @return	<code>true</code> if the listener was added,
	 * 			<code>false</code> otherwise.
	 */
	public boolean addProjectDatabaseManagerListener(
			ProjectDatabaseManagerChangeListener listener) {
		return pdmListeners.add(listener);
	}
	
	/**
	 * Removes a ProjectDatabaseManagerChangeListener from the list of
	 * listeners.
	 * 
	 * @param	listener	The listener to remove.
	 * 
	 * @return	<code>true</code> if the listener was remove,
	 * 			<code>false</code> otherwise.
	 */
	public boolean removeProjectDatabaseManagerListener(
			ProjectDatabaseManagerChangeListener listener) {
		return pdmListeners.remove(listener);
	}
	
	/**
	 * @see org.eclipse.core.resources.IResourceChangeListener#resourceChanged(org.eclipse.core.resources.IResourceChangeEvent)
	 */
	public void resourceChanged(IResourceChangeEvent event) {
		resourcesChanged(event.getDelta(),1);
	}
	
	/**
	 * Called when a resource has changed.
	 * 
	 * @param	delta	The change delta that occured.
	 * @param	indent	How far down the tree/indent this is happening.
	 * 					This is incremented as this method is recursive.
	 */
	private void resourcesChanged(IResourceDelta delta, int indent){
		oneResourceChanged(delta,indent);
		IResourceDelta[] children = delta.getAffectedChildren();
		for ( int i = 0; i < children.length; i++ ){
			resourcesChanged(children[i],indent+1);
		}
	}
	
	/**
	 * Called when a resource has changed. Closes the connection
	 * and clears the list of connections.
	 * 
	 * @param	delta	The change delta that occured.
	 * @param	indent	How far down the tree/indent this is happening.
	 * 					This is incremented as this method is recursive.
	 */
	private void oneResourceChanged(IResourceDelta delta, int indent){
		IResource resource = delta.getResource();
		if ( resource.getLocation().toString().equals(project.getLocation() + DATABASE_YML_PATH) ){
			Iterator i = connections.values().iterator();
			while( i.hasNext() ) {
				Connection conn = (Connection) i.next();
				try {
					conn.close();
				} catch (SQLException e) {
					e.printStackTrace();
				}
			}
			connections.clear();
			notifyListeners();
		}
		
	}

	/**
	 * Notifies all listeners that this ProjectDatabaseManager
	 * has changed.
	 */
	private void notifyListeners() {
		Iterator i = pdmListeners.iterator();
		while( i.hasNext() ) {
			ProjectDatabaseManagerChangeListener listener
				= (ProjectDatabaseManagerChangeListener) i.next();
			listener.projectDatabaseChanged(this);
		}
	}
	
	/**
	 * @return the project name
	 */
	public String getProjectName() {
		return project.getName();
	}

	/**
	 * @return the project
	 */
	public IProject getIProject() {
		return project;
	}

} // ProjectDatabases
