/*******************************************************************************
 * Copyright (c) 2005 RadRails.org and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.radrails.org/legal/cpl-v10.html
 *******************************************************************************/

package org.radrails.db.core;

import java.sql.Connection;
import java.sql.DatabaseMetaData;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.ResultSetMetaData;
import java.sql.SQLException;
import java.sql.Types;
import java.util.ArrayList;
import java.util.Collection;
import java.util.LinkedList;
import java.util.List;

/**
 * Manages queries and returns objects based on SQL queries. Provides simple
 * methods for getting columns, tables, etc.
 * 
 * @author mbaumbach
 * 
 * @version 0.3.0
 */
public class QueryAdministrator {

	/**
	 * Reference to the JDBC connection.
	 */
	private Connection conn;

	private static final String ORACLE_BUG = "$";

	private static final String ORACLE_PROBLEM_CHECK = "oracle";

	private boolean isOracle = false;

	/**
	 * Creates a new QueryAdministrator based on the specified connection.
	 * 
	 * @param conn
	 *            The java.sql.Connection for the database connection.
	 */
	public QueryAdministrator(Connection conn) {
		this.conn = conn;

		// ORACLE 10g potential problem hack
		if (conn != null) {
			// crude
			isOracle = conn.getClass().getName().toLowerCase().indexOf(
					ORACLE_PROBLEM_CHECK) != -1;
		}
	}

	/**
	 * Executes a SQL statement on the connection.
	 * 
	 * @param sql
	 *            The SQL statement to execute.
	 * @param fetchSize
	 *            The number of rows to return;
	 * 
	 * @return A ResultSet of the resulting data based on the execution of the
	 *         statement.
	 * 
	 * @throws SQLException
	 *             If an error occurred parsing the SQL statement.
	 */
	private ResultSet executeQuery(String sql, int fetchSize)
			throws SQLException {
		PreparedStatement statement = null;
		statement = conn.prepareStatement(sql);
		statement.setFetchSize(fetchSize);
		statement.execute();
		return statement.getResultSet();
	}

	/**
	 * Gets the tables in the database. The tables are a simple Collection of
	 * String objects for the names of the tables.
	 * 
	 * @return A Collection of table names represented as Strings.
	 */
	public Collection getTables() {
		List tables = new LinkedList();

		ResultSet rs = null;
		try {
			final String schema = isOracle ? conn.getMetaData().getUserName()
					: null;
			rs = conn.getMetaData().getTables(null, schema, null, null);
			String table = "";
			while (rs.next()) {
				table = rs.getString(3);
				if (!isPotentialOracle10gProblem(table)) {
					tables.add(table);
				}
			}
		} catch (SQLException e) {
			DatabaseLog.logError("Unable to get tables", e);
		} finally {
			if ( rs != null ) {
				closeResultSet(rs);
			}
		}
		
		return tables;
	}
	
	private void closeResultSet(ResultSet rs) {
		if (rs != null) {
			try {
				rs.close();
			} catch (Exception e) {
				DatabaseLog.logError("Unable to close result set!", e);
			}
		}
	}
	
	/**
	 * Gets the columns in a table. The columns are a simple Collection of
	 * String objects for the names of the columns.
	 * 
	 * @param table
	 *            The table to get the columns for.
	 * 
	 * @return A Collection of columns names represented as String.
	 */
	public Collection getColumns(String table) {
		List columns = new LinkedList();

		if (!isPotentialOracle10gProblem(table)) {
			ResultSet rs = null;
			try {
				rs = conn.getMetaData().getColumns(null, null, table, null);
				while (rs.next()) {
					// show columns as: name (type)
					columns.add(rs.getString(4) + " (" + rs.getString(6) + ")");
				}
			} catch (SQLException e) {
				DatabaseLog.logError("Unable to get columns", e);
			} finally {
				if ( rs != null ) {
					closeResultSet(rs);
				}
			}
		}

		return columns;
	}

	private boolean isPotentialOracle10gProblem(final String table) {
		return (isOracle && (table.indexOf(ORACLE_BUG) != -1));
	}
	
	public boolean isPrimaryKey(String column, String table) {
		boolean retVal = false;
		if ( !isOracle ) {
			try {
				DatabaseMetaData dbmd = conn.getMetaData();
				ResultSet pkrs = dbmd.getPrimaryKeys(null, null, table);
				
				while (pkrs.next()) {
					if ( column.equals(pkrs.getString(4)) ) {
						retVal = true;
						break;
					}
				}
			} catch (SQLException e) {
				DatabaseLog.logError("Unable to get database metadata", e);
			}
		}
		return retVal;
	}

	public boolean isForeignKey(String column, String table) {
		boolean retVal = false;
		if ( !isOracle ) {
			try {
				DatabaseMetaData dbmd = conn.getMetaData();
				ResultSet fkrs = dbmd.getImportedKeys(null, null, table);
	
				while (fkrs.next()) {
					if ( column.equals(fkrs.getString(8)) ) {
						retVal = true;
						break;
					}
				}
			} catch (SQLException e) {
				DatabaseLog.logError("Unable to get database metadata", e);
			}
		}
		return retVal;
	}

	/**
	 * Executes a SQL query and returns a QueryResult object.
	 * 
	 * @param sql
	 *            The SQL statement to execute.
	 * @param fetchSize
	 *            The number of rows to fetch.
	 * 
	 * @return A QueryResult containing the data returned.
	 * 
	 * @throws SQLException
	 *             If the statement is incorrect syntax or failed to execute.
	 */
	public QueryResult execute(String sql, int fetchSize) throws SQLException {
		List rows = new LinkedList();
		QueryResult result = null;
		// Get the data
		ResultSet rs = executeQuery(sql, fetchSize);
		if (rs != null) {
			ResultSetMetaData rsmd = rs.getMetaData();
			int colCount = rsmd.getColumnCount();
			while (rs.next()) {
				List row = new LinkedList();
				for (int i = 1; i <= colCount; i++) {
					Object o = rs.getObject(i);
					if (o instanceof Integer && !rsmd.isSigned(i)) {
						Integer integ = (Integer) o;
						if (rsmd.getColumnType(i) == Types.SMALLINT) {
							row.add(new Integer(integ.shortValue()));
						} else {
							row.add(new Integer(integ.intValue()));
						}
					} else if (o instanceof Long && !rsmd.isSigned(i)) {
						Long lon = (Long) o;
						row.add(new Long(lon.intValue()));
					} else if (o instanceof Double && !rsmd.isSigned(i)) {
						Double doub = (Double) o;
						row.add(new Double(doub.intValue()));
					} else if (o instanceof Short && !rsmd.isSigned(i)) {
						Short shor = (Short) o;
						row.add(new Integer(shor.intValue()));
					} else {
						row.add(o);
					}
				}
				rows.add(row);
			}

			// Get the column names
			List columnNames = new LinkedList();
			for (int i = 1; i <= colCount; i++) {
				columnNames.add(rsmd.getColumnName(i));
			}

			// Create the query result based on the data and column names
			result = new QueryResult(columnNames, rows);
		} else {
			result = new QueryResult(new ArrayList(), new LinkedList());
		}
		return result;
	}
	
	/**
	 * Return the raw JDBC ResultSet for a query.
	 * 
	 * @param sql the statement to execute
	 * @return a jdbc result set
	 */
	public ResultSet executeRaw(String sql, int rowLimit) throws SQLException {
		return executeQuery(sql, 0);
	}

	/**
	 * Executes a SQL query and returns a QueryResult object.
	 * 
	 * @param sql
	 *            The SQL statement to execute.
	 * 
	 * @return A QueryResult containing the data returned.
	 * 
	 * @throws SQLException
	 *             If the statement is incorrect syntax or failed to execute.
	 */
	public QueryResult execute(String sql) throws SQLException {
		return execute(sql, 0);
	}

	/**
	 * Executes a SQL statement that can modify the table.
	 * 
	 * @param sql
	 *            The SQL statement to execute.
	 * 
	 * @throws SQLException
	 *             If an error occurred during execution.
	 */
	public void executeStatement(String sql) throws SQLException {
		PreparedStatement statement = null;
		statement = conn.prepareStatement(sql);
		statement.execute();
	}

} // QueryAdministrator
