/*******************************************************************************
 * Copyright (c) 2005 RadRails.org and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.radrails.org/legal/cpl-v10.html
 *******************************************************************************/

package org.radrails.db.core;

import java.io.BufferedReader;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.PrintWriter;
import java.io.Reader;
import java.io.StringWriter;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;

/**
 * Parses a YML file and returns objects which describe how to connect to the
 * available databases.
 * 
 * @author hgoale
 * 
 * @version 0.1
 * @deprecated
 */
public class YMLFile {

	private static final char YML_COMMENT = '#';

	private static final String YML_SEPARATOR = ":";

	private static final String DB_DEVELOPMENT_STR = "development"
			+ YML_SEPARATOR;

	private static final String DB_TEST_STR = "test" + YML_SEPARATOR;

	private static final String DB_PRODUCTION_STR = "production"
			+ YML_SEPARATOR;

	private static final String DB_NODB_STR = "nodatabase";

	private static final String KEY_ADAPTER = "adapter";

	private static final String KEY_DATABASE = "database";

	private static final String KEY_HOST = "host";

	private static final String KEY_USERNAME = "username";

	private static final String KEY_PASSWORD = "password";

	private static final String KEY_PORT = "port";

	private static final String KEY_SSLCERT = "sslcert";

	private static final String KEY_SSLCAPATH = "sslcapath";

	private static final String KEY_SSLCIPHER = "sslcipher";

	private static final String KEY_SOCKET = "socket";

	private static final String KEY_SSLKEY = "sslkey";

	private static final String KEY_SCHEMA_ORDER = "schema_order";

	private static final String KEY_DBFILE = "dbfile";

	private DatabaseDescriptor dbDevelopment;

	private DatabaseDescriptor dbTest;

	private DatabaseDescriptor dbProduction;

	private Collection listeners;

	/**
	 * Default constructor.
	 */
	public YMLFile() {
		listeners = new ArrayList();

		dbDevelopment = new DatabaseDescriptor(this);
		dbTest = new DatabaseDescriptor(this);
		dbProduction = new DatabaseDescriptor(this);
	}

	/**
	 * Copy constructor.
	 * 
	 * @param	file	The YMLFile to create a new YMLFile from.
	 */
	public YMLFile(YMLFile file) {
		listeners = new ArrayList(file.listeners);

		dbDevelopment = new DatabaseDescriptor(this, file.dbDevelopment);
		dbTest = new DatabaseDescriptor(this, file.dbTest);
		dbProduction = new DatabaseDescriptor(this, file.dbProduction);
	}

	/**
	 * Add a listener.
	 * 
	 * @param	listener	The IYMLFileChangeListener to add.
	 */
	public void addListener(IYMLFileChangeListener listener) {
		listeners.add(listener);
	}

	/**
	 * Remove a listener.
	 * 
	 * @param	listener	The IYMLFileChangeListener to remove.
	 */
	public void removeListener(IYMLFileChangeListener listener) {
		listeners.remove(listener);
	}

	/**
	 * Notify all listenters that the file contents have changed.
	 */
	protected void fireChangeEvent() {
		Iterator i = listeners.iterator();
		while (i.hasNext()) {
			IYMLFileChangeListener listener = (IYMLFileChangeListener) i.next();
			listener.YMLFileChanged(new YMLFile(this));
		}
	}

	/**
	 * Gets the dbDevelopment.
	 * 
	 * @return Returns the dbDevelopment, or <code>null</code> if no
	 *         development database is defined.
	 */
	public DatabaseDescriptor getDbDevelopment() {
		return dbDevelopment;
	}

	/**
	 * Gets the dbProduction.
	 * 
	 * @return Returns the dbProduction, or <code>null</code> if no
	 *         development database is defined.
	 */
	public DatabaseDescriptor getDbProduction() {
		return dbProduction;
	}

	/**
	 * Gets the dbTest.
	 * 
	 * @return Returns the dbTest, or <code>null</code> if no development
	 *         database is defined.
	 */
	public DatabaseDescriptor getDbTest() {
		return dbTest;
	}

	/**
	 * Parses a YML file and sets the DatabaseDescriptors for this class.
	 * 
	 * @throws FileNotFoundException
	 *             If the file could not be found.
	 * @throws IOException
	 *             If the file could not be parsed.
	 */
	public void load(Reader reader) throws IOException {
		// Reset database descriptors
		dbDevelopment = new DatabaseDescriptor(this);
		dbTest = new DatabaseDescriptor(this);
		dbProduction = new DatabaseDescriptor(this);
		
		// Read the file in line by line
		BufferedReader in = new BufferedReader(reader);
		String r;
		DatabaseDescriptor current = null;
		while ((r = in.readLine()) != null) {
			// Trim the whitespace
			r = r.trim();

			// Determine if this is one of the databases
			String db = getDatabase(r);
			if (db.equals(DB_NODB_STR)) {
				// Verify we have a database and the current line isn't a
				// comment or
				// whitespace
				if (current != null && !r.equals("")
						&& r.charAt(0) != YML_COMMENT) {
					// This must be an attribute, set it in the current database
					setAttribute(r, current);
				}
			} else {
				if (db.equals(DB_DEVELOPMENT_STR)) {
					// dbDevelopment = new DatabaseDescriptor(this);
					current = getDbDevelopment();
				} else if (db.equals(DB_TEST_STR)) {
					// dbTest = new DatabaseDescriptor(this);
					current = getDbTest();
				} else if (db.equals(DB_PRODUCTION_STR)) {
					// dbProduction = new DatabaseDescriptor(this);
					current = getDbProduction();
				}
			}
		}
		getDbDevelopment().changed();
		getDbTest().changed();
		getDbProduction().changed();
	}

	/**
	 * Sets the current line as an attribute in the database if it is a valid
	 * attribute.
	 * 
	 * @param line
	 *            The current line.
	 * @param db
	 *            The current database to save it to.
	 */
	private void setAttribute(String line, DatabaseDescriptor db) {
		// First split on the first colon
		String[] splitLine = line.split(":", 2);
		if (splitLine.length == 2) {
			// Trim the white space off
			String key = splitLine[0].trim();
			String value = splitLine[1].trim();

			// Determine if the key is a valid key
			if (key.equalsIgnoreCase(KEY_ADAPTER)) {
				db.setAdapter(value);
			} else if (key.equalsIgnoreCase(KEY_DATABASE)) {
				db.setDatabase(value);
			} else if (key.equalsIgnoreCase(KEY_HOST)) {
				db.setHost(value);
			} else if (key.equalsIgnoreCase(KEY_USERNAME)) {
				db.setUsername(value);
			} else if (key.equalsIgnoreCase(KEY_PASSWORD)) {
				db.setPassword(value);
			} else if ( key.equalsIgnoreCase(KEY_PORT) ) {
				db.setPort(value);
			} else if ( key.equalsIgnoreCase(KEY_SSLCERT) ) {
				db.setSSLCert(value);
			} else if ( key.equalsIgnoreCase(KEY_SSLCAPATH) ) {
				db.setSSLCapath(value);
			} else if ( key.equalsIgnoreCase(KEY_SSLCIPHER) ) {
				db.setSSLCipher(value);
			} else if ( key.equalsIgnoreCase(KEY_SOCKET) ) {
				db.setSocket(value);
			} else if ( key.equalsIgnoreCase(KEY_SSLKEY) ) {
				db.setSSLKey(value);
			} else if ( key.equalsIgnoreCase(KEY_SCHEMA_ORDER) ) {
				db.setSchemaOrder(value);
			} else if ( key.equalsIgnoreCase(KEY_DBFILE) ) {
				db.setDBFile(value);
			}
		}
	}

	/**
	 * Checks if the current line being parsed is a database.
	 * 
	 * @param line
	 *            The current line to check.
	 * 
	 * @return Returns the type of database if it is a database, otherwise
	 *         returns DB_NODB_STR.
	 */
	private String getDatabase(String line) {
		String dbType = DB_NODB_STR;

		if (line.equalsIgnoreCase(DB_DEVELOPMENT_STR)) {
			dbType = DB_DEVELOPMENT_STR;
		} else if (line.equalsIgnoreCase(DB_TEST_STR)) {
			dbType = DB_TEST_STR;
		} else if (line.equalsIgnoreCase(DB_PRODUCTION_STR)) {
			dbType = DB_PRODUCTION_STR;
		}

		return dbType;
	}

	/**
	 * Gets the contents of what the YML file should be.
	 * 
	 * @return A String of the contents of the file.
	 * 
	 * @throws FileNotFoundException
	 *             If the file can not be found.
	 * @throws IOException
	 *             If an error occurred during the save process.
	 * @deprecated
	 */

	public String asText() {
		StringWriter text = new StringWriter();

		PrintWriter out = new PrintWriter(text);

		writeDatabaseDescriptor(out, getDbDevelopment(), DB_DEVELOPMENT_STR);
		out.println();
		writeDatabaseDescriptor(out, getDbTest(), DB_TEST_STR);
		out.println();
		writeDatabaseDescriptor(out, getDbProduction(), DB_PRODUCTION_STR);

		out.close();

		return text.getBuffer().toString();
	}

	/**
	 * Writes a DatabaseDescriptor object to a PrintWriter's output.
	 * 
	 * @param out
	 *            The PrintWriter to use.
	 * @param db
	 *            The DatabaseDescriptor to write.
	 * @param dbName
	 *            The name of the database (development:, test:, production:,
	 *            etc.)
	 * @deprecated
	 */
	private void writeDatabaseDescriptor(PrintWriter out,
			DatabaseDescriptor db, String dbName) {
		out.println(dbName);
		writeDBAttribute(out, KEY_ADAPTER, db.getAdapter());
		writeDBAttribute(out, KEY_DATABASE, db.getDatabase());
		writeDBAttribute(out, KEY_HOST, db.getHost());
		writeDBAttribute(out, KEY_USERNAME, db.getUsername());
		writeDBAttribute(out, KEY_PASSWORD, db.getPassword());
		writeDBAttribute(out, KEY_PORT, db.getPort());
		writeDBAttribute(out, KEY_SSLCERT, db.getSSLCert());
		writeDBAttribute(out, KEY_SSLCAPATH, db.getSSLCapath());
		writeDBAttribute(out, KEY_SSLCIPHER, db.getSSLCipher());
		writeDBAttribute(out, KEY_SOCKET, db.getSocket());
		writeDBAttribute(out, KEY_SSLKEY, db.getSSLKey());
		writeDBAttribute(out, KEY_SCHEMA_ORDER, db.getSchemaOrder());
		writeDBAttribute(out, KEY_DBFILE, db.getDBFile());
	}

	/**
	 * Writes the attribute only if it exists (Not <code>null</code> or
	 * empty).
	 * 
	 * @param out
	 *            The PrintWriter to write to.
	 * @param key
	 *            The attribute key.
	 * @param attribute
	 *            The attribute to write.
	 * @deprecated
	 */
	private void writeDBAttribute(PrintWriter out, String key, String attribute) {
		if ( attribute == null ) {
			attribute = "";
		}
		out.println("  " + key + ": " + attribute);
	}

} // YMLParser
