/*******************************************************************************
 * Copyright (c) 2005 RadRails.org and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.radrails.org/legal/cpl-v10.html
 *******************************************************************************/

package org.radrails.db.internal.ui;

import java.io.File;
import java.io.FileReader;
import java.sql.Connection;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.Iterator;
import java.util.Observable;
import java.util.Observer;

import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.DoubleClickEvent;
import org.eclipse.jface.viewers.IDoubleClickListener;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.part.ViewPart;
import org.radrails.db.core.DatabaseDescriptor;
import org.radrails.db.core.DatabaseLog;
import org.radrails.db.core.DatabaseManager;
import org.radrails.db.core.DatabaseYml;
import org.radrails.db.core.IDatabaseConstants;
import org.radrails.db.core.ProjectDatabaseManager;
import org.radrails.db.core.ProjectDatabaseManagerChangeListener;
import org.radrails.db.core.QueryAdministrator;
import org.radrails.db.internal.ui.editors.dbtable.DbTableEditorInput;

/**
 * Provides a view for the IDE to display databases and their tables/columns.
 * 
 * @author Kyle
 * @author mbaumbach
 * 
 * @version 0.3.1
 */
public class DataNavigator extends ViewPart implements
		ProjectDatabaseManagerChangeListener, Observer {

	private static TreeViewer dataTree;

	private Collection projects = Collections
			.synchronizedCollection(new ArrayList());

	private DataContentProvider contentProvider;

	/**
	 * Creates a new DataNavigator view.
	 */
	public DataNavigator() {
		super();
		DatabaseManager.getInstance().addObserver(this);
		contentProvider = new DataContentProvider();
	}

	/**
	 * Gets a reference to the TreeViewer for this Data Navigator.
	 * 
	 * @return The TreeViewer for this class.
	 */
	public static TreeViewer getTreeViewer() {
		return dataTree;
	}

	/**
	 * @see org.eclipse.ui.IWorkbenchPart#createPartControl(org.eclipse.swt.widgets.Composite)
	 */
	public void createPartControl(Composite parent) {
		dataTree = new TreeViewer(parent, SWT.SINGLE);
		dataTree.setContentProvider(contentProvider);
		dataTree.setLabelProvider(new DataLabelProvider());
		dataTree.setInput(getInitialInput());
		dataTree.collapseAll();
		dataTree.addDoubleClickListener(new IDoubleClickListener() {

			public void doubleClick(DoubleClickEvent event) {
				StructuredSelection selection = (StructuredSelection) event
						.getSelection();
				Object element = selection.getFirstElement();
				if (element instanceof org.radrails.db.internal.ui.Database) {
					Database database = (Database) element;
					database.clear();
					Project project = (Project) database.getParent();
					try {
						constructDatabaseTree(project
								.getProjectDatabaseManager(), database);
						refreshDataTree();
					} catch (Exception e) {
						MessageDialog
								.openError(getViewSite().getShell(),
										"Database Error",
										"Unable to connect to database, see log for more info.");
						DatabaseLog.logError(
								"Unable to construct database tree", e);
					}
				} else if (element instanceof org.radrails.db.internal.ui.Table) {
					Table table = (Table) element;
					Database database = (Database) table.getParent();
					openEditor(table.getName(), database);
				} 
				else if (element instanceof org.radrails.db.internal.ui.Project) {
					Project proj = (Project) element;
					proj.clearDatabases();
					try {
						constructDatabaseList(proj);
						refreshDataTree();
					} catch (Exception e) {
						MessageDialog
								.openError(getViewSite().getShell(),
										"Database Error",
										"Unable to connect to database, see log for more info.");
						DatabaseLog.logError(
								"Unable to construct database tree", e);
					}
				}
			}

		});
	}

	/**
	 * @see org.eclipse.ui.IWorkbenchPart#dispose()
	 */
	public void dispose() {
		Collection pdms = DatabaseManager.getInstance()
				.getAllProjectDatabaseManagers();
		Iterator i = pdms.iterator();
		while (i.hasNext()) {
			ProjectDatabaseManager pdm = (ProjectDatabaseManager) i.next();
			pdm.removeProjectDatabaseManagerListener(this);
		}
	}

	/**
	 * Opens the Result editor when a table is clicked.
	 * 
	 * @param tableName
	 *            The table to get all rows for.
	 * @param database
	 *            The database the table resides in.
	 */
	private void openEditor(String tableName, Database database) {
		try {
			DbTableEditorInput input = new DbTableEditorInput(tableName,
					database);
			getViewSite().getPage().openEditor(input,
					"org.radrails.db.ui.DbTableEditor");
		} catch (PartInitException e) {
			DatabaseLog.logError("Could not open query editor", e);
		}
	}

	/**
	 * @see org.eclipse.ui.IWorkbenchPart#setFocus()
	 */
	public void setFocus() {
	}

	/**
	 * Constructs the initial tree when this view is displayed.
	 * 
	 * @return An Object array of the data to be put into the tree.
	 */
	public Object[] getInitialInput() {
		Collection projectDatabaseManagers = DatabaseManager.getInstance()
				.getAllProjectDatabaseManagers();

		Iterator p = projectDatabaseManagers.iterator();
		while (p.hasNext()) {
			ProjectDatabaseManager pdm = (ProjectDatabaseManager) p.next();
			pdm.addProjectDatabaseManagerListener(this);
			Project project = new Project(pdm);
			projects.add(project);
			contentProvider.add(project);
		}

		return projects.toArray();
	}

	/**
	 * Constructs a database object using a ProjectDatabaseManager.
	 * 
	 * @param pdm
	 *            The ProjectDatabaseManager to use.
	 * @param database
	 *            The database to use.
	 * 
	 * @return A Database with all tables and columns.
	 * 
	 * @throws Exception
	 *             If an error occurred while constructing the database tree.
	 */
	private Database constructDatabaseTree(ProjectDatabaseManager pdm,
			Database database) throws Exception {
		Connection developmentConnection = pdm
				.getConnection(database.getName());
		QueryAdministrator query = new QueryAdministrator(developmentConnection);
		Collection tables = query.getTables();
		Iterator i = tables.iterator();
		while (i.hasNext()) {
			String tableName = (String) i.next();
			Table table = new Table(tableName, database);
			Collection columns = query.getColumns(tableName);
			Iterator j = columns.iterator();
			while (j.hasNext()) {
				String column = (String) j.next();
				boolean isPk = query.isPrimaryKey(column, tableName);
				boolean isFk = query.isForeignKey(column, tableName);
				table.addColumn(new Column(column, table, isPk, isFk));
			}
			database.addTable(table);
		}
		return database;
	}

	private void constructDatabaseList(Project proj) throws Exception {
		File projFile = proj.getProject().getLocation().toFile();
		File databaseYML = new File(projFile + "/config/database.yml");
		FileReader reader = new FileReader(databaseYML);
		DatabaseYml yml = new DatabaseYml();
		yml.load(reader);

		Iterator i = yml.getDescriptors().iterator();
		while (i.hasNext()) {
			DatabaseDescriptor dd = (DatabaseDescriptor) i.next();
			Database db = new Database(dd.getName(), proj);
			proj.addDatabase(db);
		}
	}

	/**
	 * @see org.radrails.db.core.ProjectDatabaseManagerChangeListener#projectDatabaseChanged(org.radrails.db.core.ProjectDatabaseManager)
	 */
	public void projectDatabaseChanged(ProjectDatabaseManager manager) {
		String projectName = manager.getProjectName();
		Iterator i = projects.iterator();
		while (i.hasNext()) {
			Project project = (Project) i.next();
			if (projectName.equals(project.getName())) {
				project.clearDatabases();
				Database development = new Database(
						IDatabaseConstants.ENV_DEVELOPMENT, project);
				Database test = new Database(IDatabaseConstants.ENV_TEST,
						project);
				Database production = new Database(
						IDatabaseConstants.ENV_PRODUCTION, project);
				try {
					constructDatabaseTree(manager, development);
				} catch (Exception e) {
					// Do nothing. Fail silently. No need to log this.
				}
				try {
					constructDatabaseTree(manager, test);
				} catch (Exception e) {
					// Do nothing. Fail silently. No need to log this.
				}
				try {
					constructDatabaseTree(manager, production);
				} catch (Exception e) {
					// Do nothing. Fail silently. No need to log this.
				}
				project.addDatabase(development);
				project.addDatabase(test);
				project.addDatabase(production);
				refreshDataTree();
			}
		}
	}

	/**
	 * @see java.util.Observer#update(java.util.Observable, java.lang.Object)
	 */
	public void update(Observable arg0, Object arg1) {
		checkForRemovedProject();
		checkForAddedProject();
		refreshDataTree();
	}

	/**
	 * Refreshes the data tree.
	 */
	private void refreshDataTree() {
		Display display = getViewSite().getShell().getDisplay();
		display.syncExec(new Runnable() {

			public void run() {
				try {
					dataTree.refresh();
				} catch (Exception e) {
					DatabaseLog.logWarning("Could not refresh data tree", e);
				}

			}

		});
	}

	/**
	 * Checks if a project was removed and if it should be removed from the data
	 * navigator.
	 */
	private void checkForRemovedProject() {
		Collection pdms = DatabaseManager.getInstance()
				.getAllProjectDatabaseManagers();
		Iterator currentProjectsIterator = projects.iterator();
		while (currentProjectsIterator.hasNext()) {
			Project currentProject = ((Project) currentProjectsIterator.next());
			String currentProjectName = currentProject.getName();
			Iterator newProjectsIterator = pdms.iterator();
			boolean found = false;
			while (newProjectsIterator.hasNext()) {
				ProjectDatabaseManager manager = (ProjectDatabaseManager) newProjectsIterator
						.next();
				String newProjectName = manager.getProjectName();
				if (currentProjectName.equals(newProjectName)) {
					found = true;
				}
			}
			if (!found) {
				projects.remove(currentProject);
				contentProvider.remove(currentProject.getName());
				refreshDataTree();
			}
		}
	}

	/**
	 * Checks if a project was added and if it should be added to the data
	 * navigator.
	 */
	private void checkForAddedProject() {
		Collection pdms = DatabaseManager.getInstance()
				.getAllProjectDatabaseManagers();
		Iterator i = pdms.iterator();
		while (i.hasNext()) {
			ProjectDatabaseManager pdm = (ProjectDatabaseManager) i.next();
			String projectName = pdm.getProjectName();
			Iterator j = projects.iterator();
			boolean found = false;
			while (j.hasNext()) {
				Project p = (Project) j.next();
				if (projectName.equals(p.getName())) {
					found = true;
				}
			}
			if (!found) {
				// Add
				Project project = new Project(pdm);
				pdm.addProjectDatabaseManagerListener(this);
				Database development = new Database(
						IDatabaseConstants.ENV_DEVELOPMENT, project);
				Database test = new Database(IDatabaseConstants.ENV_TEST,
						project);
				Database production = new Database(
						IDatabaseConstants.ENV_PRODUCTION, project);
				project.addDatabase(development);
				project.addDatabase(test);
				project.addDatabase(production);
				projects.add(project);
				contentProvider.add(project);
				projectDatabaseChanged(pdm);
			}
		}
	}

} // DataNavigator
