package org.radrails.db.internal.ui.editors.sql;
/*
 * "The Java Developer's Guide to Eclipse"
 *   by D'Anjou, Fairbrother, Kehn, Kellerman, McCarthy
 * 
 * (C) Copyright International Business Machines Corporation, 2003, 2004. 
 * All Rights Reserved.
 * 
 * Code or samples provided herein are provided without warranty of any kind.
 */
import java.util.ArrayList;
import java.util.List;

import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.jface.text.rules.EndOfLineRule;
import org.eclipse.jface.text.rules.IRule;
import org.eclipse.jface.text.rules.IToken;
import org.eclipse.jface.text.rules.MultiLineRule;
import org.eclipse.jface.text.rules.SingleLineRule;
import org.eclipse.jface.text.rules.WhitespaceRule;
import org.eclipse.jface.text.rules.WordRule;
import org.eclipse.ui.editors.text.EditorsUI;
import org.eclipse.ui.texteditor.AbstractTextEditor;
import org.eclipse.ui.texteditor.ChainedPreferenceStore;
import org.radrails.db.internal.ui.DbUIPlugin;
import org.radrails.db.internal.ui.editors.sql.util.SQLWhiteSpaceDetector;
import org.radrails.db.internal.ui.editors.sql.util.SQLWordDetector;
import org.radrails.rails.internal.ui.editors.AbstractManagedRuleBasedScanner;

/**
 * The SQLCodeScanner is a RuleBaseScanner.This class finds SQL comments and
 * keywords, as the user edits the document. It is "programmed" with a sequence
 * of rules that evaluates and returns the offset and the length of the last
 * found token.
 */
public class SQLCodeScanner extends AbstractManagedRuleBasedScanner
    implements
      ISQLSyntax {

	private static final String[] COLOR_KEYS = {ISqlEditorColorConstants.KEYWORD, ISqlEditorColorConstants.MULTILINE_COMMENT,
		ISqlEditorColorConstants.SINGLELINE_COMMENT, ISqlEditorColorConstants.STRING, ISqlEditorColorConstants.TYPE,
		AbstractTextEditor.PREFERENCE_COLOR_FOREGROUND};

  /**
   * Constructor for SQLCodeScanner. The SQLCodeScanner, is a RuleBaseScanner.
   * The code scanner creates tokens for keywords, types, and constants. The
   * token is constructed with a TextAttribute. The TextAttribute is constructed
   * with a color and font. A list of rules with the corresponding token are
   * created. The method ends with setting the scanner's set of rules
   */
  public SQLCodeScanner() {
	  super();
	  IToken keyword = getToken(ISqlEditorColorConstants.KEYWORD);
	  IToken type = getToken(ISqlEditorColorConstants.TYPE);
	  IToken string = getToken(ISqlEditorColorConstants.STRING);
	  IToken singlelineComment = getToken(ISqlEditorColorConstants.SINGLELINE_COMMENT);
	  IToken multilineComment = getToken(ISqlEditorColorConstants.MULTILINE_COMMENT);
	  IToken def = getToken(AbstractTextEditor.PREFERENCE_COLOR_FOREGROUND);

    setDefaultReturnToken(def);
    List rules = new ArrayList();

    // Add rule for single line comments.
    rules.add(new EndOfLineRule("//", singlelineComment));
    rules.add(new EndOfLineRule("--", singlelineComment));
    
    // Add rules for multi-line comments
    rules.add(new MultiLineRule("/*", "*/", multilineComment, (char) 0,
        true));

    // Add rule for strings and character constants.
    rules.add(new SingleLineRule("\"", "\"", string, '\\'));
    rules.add(new SingleLineRule("'", "'", string, '\\'));

    // Add generic whitespace rule.
    rules.add(new WhitespaceRule(new SQLWhiteSpaceDetector()));

    // Add word rule for keywords, types, and constants.
    WordRule wordRule = new WordRule(new SQLWordDetector(),
        def);
    for (int i = 0; i < reservedwords.length; i++)
      wordRule.addWord(reservedwords[i], keyword);
    for (int i = 0; i < types.length; i++)
      wordRule.addWord(types[i], type);
    for (int i = 0; i < constants.length; i++)
      wordRule.addWord(constants[i], type);
    rules.add(wordRule);

    IRule[] result = new IRule[rules.size()];
    rules.toArray(result);
    setRules(result);

  }

	protected String[] getColorKeys() {
		return COLOR_KEYS;
	}

	protected IPreferenceStore getPreferenceStore() {
		IPreferenceStore[] prefStores = new IPreferenceStore[2];
		prefStores[0] = DbUIPlugin.getInstance().getPreferenceStore();
		prefStores[1] = EditorsUI.getPreferenceStore();
		return new ChainedPreferenceStore(prefStores);
	}

}