/*******************************************************************************
 * Copyright (c) 2005 RadRails.org and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.radrails.org/legal/cpl-v10.html
 *******************************************************************************/

package org.radrails.db.internal.ui.editors.sql.util;

import java.io.BufferedReader;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStreamReader;
import java.sql.Connection;
import java.sql.SQLException;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.runtime.CoreException;
import org.radrails.db.core.DatabaseManager;
import org.radrails.db.core.ProjectDatabaseManager;
import org.radrails.db.core.QueryAdministrator;

/**
 * Class that helps to parse a String of SQL statements executing them one by
 * one and handling errors gracefully.
 * 
 * @author	mbaumbach
 * @author  Kyle
 *
 * @version	0.5.1
 */
public class SQLExecuteHelper {

	/**
	 * Reference to the only instance.
	 */
	private static SQLExecuteHelper instance;
	
	/**
	 * Private constructor since this is a singleton. Use the getInstance()
	 * method to get an instance of this class.
	 */
	private SQLExecuteHelper() {}
	
	/**
	 * Gets an instance of this class.
	 * 
	 * @return	A SQLExecuteHelper object.
	 */
	public static SQLExecuteHelper getInstance() {
		if ( instance == null ) {
			instance = new SQLExecuteHelper();
		}
		return instance;
	}
	
	/**
	 * Executes the statements 
	 * 
	 * @param	file	The SQL file to execute.
	 * @param	env	The environment to run from the project's file.
	 * 
	 * @return	<code>true</code> if the SQL file executed successfully,
	 * 			<code>false</code> otherwise.
	 * 
	 * @throws	FileNotFoundException	If the file could not found.
	 * @throws	SQLException	If an error occurred while executing the SQL statement.
	 * @throws	InstantiationException	If the database driver could not be instantiated.
	 * @throws	IllegalAccessException	If the user does not have access to the database.
	 * @throws	IOException	If an I/O exception occurs. 
	 * @throws	CoreException	If an exception happens within Eclipse. 
	 */
	public boolean runSQLFile(IFile file, String env)
		throws FileNotFoundException, SQLException, InstantiationException, IllegalAccessException, IOException, CoreException {
		IProject project = (IProject) file.getProject();
		ProjectDatabaseManager pdm
			= DatabaseManager.getInstance().getProjectDatabaseManager(project);
		BufferedReader in = new BufferedReader(new InputStreamReader(file.getContents()));;
		StringBuffer buffer = new StringBuffer();
		String r;
		boolean inBlockComment = false;
		while( (r = in.readLine()) != null ) {
			// check for block comments
			if(r.startsWith("/*") && !inBlockComment) {
				inBlockComment = true;
			}
			else if(r.startsWith("*/") && inBlockComment) {
				inBlockComment = false;
			}
			// check for single line block comments
			else if(r.endsWith("*/") && inBlockComment) {
				inBlockComment = false;
			}
			else if ( (r.length() > 0) && (!r.trim().startsWith("--")) && !inBlockComment ){
				buffer.append(r);
			}
		}
		in.close();
		
		return executeStatements(pdm.getConnection(env), buffer.toString());
	}
	
	/**
	 * Executes a string of statements using the specified connection.
	 * 
	 * @param	conn	The connection to use.
	 * @param	statements	The statements to execute separated by newlines.
	 * 
	 * @return	<code>true</code> if the statements executed successfully,
	 * 			<code>false</code> otherwise.
	 * 
	 * @throws	SQLException	If an error occurred while executing the statements.
	 */
	private boolean executeStatements(Connection conn, String statements) throws SQLException {
		boolean successful = false;
		
		// Execute each statement using the QueryAdministrator and Connection
		QueryAdministrator query = new QueryAdministrator(conn);
		String[] statementArray = parseStatementsString(statements);
		for( int i = 0; i < statementArray.length; i++ ) {
			query.executeStatement(statementArray[i]);
		}
		
		return successful;
	}
	
	/**
	 * Parses a SQL file and returns an array of Strings representing each query.
	 * 
	 * @param	statements	The contents of a SQL file.
	 * 	
	 * @return	An array of Strings representing each query in the statements string.
	 */
	private String[] parseStatementsString(String statements) {
		statements = statements.replaceAll("\n", "");
		return statements.split(";");
	}
	
} // SQLExecuteHelper