/*******************************************************************************
 * Copyright (c) 2006 RadRails.org and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.radrails.org/legal/cpl-v10.html
 *******************************************************************************/

package org.radrails.rails.core.gems;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;
import java.io.Reader;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;

import javax.xml.parsers.FactoryConfigurationError;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.parsers.SAXParserFactory;

import org.radrails.rails.core.RailsLog;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;
import org.xml.sax.XMLReader;

/**
 * Manager for a set of Ruby gem descriptors. Loads and saves data from XML,
 * provides access to Gem descriptor objects. Subclasses specify the location of
 * the XML file.
 * 
 * @author mkent
 * @deprecated
 * 
 */
public abstract class GemsManager {

	protected Collection fGems;

	protected Gem fSelectedGem;

	protected Collection fListeners;

	protected GemsManager() {
		fListeners = new ArrayList();
	}

	/**
	 * Sets the current gems. Saves the gems to file.
	 * 
	 * @param gems
	 *            the gems
	 */
	public void setGems(Collection gems) {
		fGems = gems;
		saveGems();
	}

	/**
	 * Sets the currently selected gem to use. If the gem is different, saves
	 * the gems to file.
	 * 
	 * @param gem
	 *            the selected gem
	 */
	public void setSelectedGem(Gem gem) {
		if (fSelectedGem != gem) {
			fSelectedGem = gem;
			saveGems();
			Iterator i = fListeners.iterator();
			while(i.hasNext()) {
				IGemsEventListener gel = (IGemsEventListener) i.next();
				gel.gemSelected(gem);
			}
		}
	}

	/**
	 * Returns all of the gems on the system. Lazy loads from file.
	 * 
	 * @return the gems
	 */
	public Collection getGems() {
		if (fGems == null) {
			loadGems();
		}
		return fGems;
	}

	/**
	 * Returns the currently selected gem. Lazy loads from file.
	 * 
	 * @return the selected gem
	 */
	public Gem getSelectedGem() {
		if (fSelectedGem == null) {
			loadGems();
		}
		return fSelectedGem;
	}

	/**
	 * Add a new gem to the system. Saves the config.
	 * 
	 * @param gem
	 *            the new gem
	 */
	public void addGem(Gem gem) {
		fGems.add(gem);
		saveGems();
	}

	/**
	 * Adds a listener to this gems manager.
	 * 
	 * @param l
	 *            the listener to add
	 */
	public void addListener(IGemsEventListener l) {
		fListeners.add(l);
	}

	/**
	 * Removes a listener from this gems manager.
	 * 
	 * @param l
	 *            the listener to remove
	 */
	public void removeListener(IGemsEventListener l) {
		fListeners.remove(l);
	}

	/**
	 * Returns the location of the xml config file for the runtime. The file is
	 * stored in the plug-in state location.
	 * 
	 * @return the config file
	 */
	protected abstract File getConfigFile();

	/**
	 * Load the configuration file and write the gems out.
	 */
	protected void saveGems() {
		try {
			File f = getConfigFile();
			PrintWriter out = new PrintWriter(new FileWriter(f));
			writeXML(out);
			out.close();
		} catch (FileNotFoundException e) {
			RailsLog.logError("Rails gems config file not found", e);
		} catch (IOException e) {
			RailsLog.logError("Error opening Rails gems config", e);
		}
	}

	/**
	 * Loads the latest gems from the config file.
	 */
	protected void loadGems() {
		try {
			Reader fileReader = new FileReader(getConfigFile());
			XMLReader reader = SAXParserFactory.newInstance().newSAXParser()
					.getXMLReader();
			GemsContentHandler handler = new GemsContentHandler();
			reader.setContentHandler(handler);
			reader.parse(new InputSource(fileReader));

			fGems = handler.getGems();
			fSelectedGem = handler.getSelectedGem();
		} catch (FileNotFoundException e) {
			// This is okay, will get thrown if no config exists yet
		} catch (SAXException e) {
			RailsLog.logError("Error parsing Rails gem config file", e);
		} catch (ParserConfigurationException e) {
			RailsLog.logError("Error configuring XML parser", e);
		} catch (FactoryConfigurationError e) {
			RailsLog.logError("Error configuring parser factory", e);
		} catch (IOException e) {
			RailsLog.logError("Error reading Rails gem config file", e);
		}
	}

	/**
	 * Writes the current gems out in XML format.
	 * 
	 * @param out
	 *            the writer to use
	 */
	protected void writeXML(PrintWriter out) {
		out.println("<?xml version=\"1.0\" encoding=\"UTF-8\"?>");
		out.println("<gems>");
		Iterator i = fGems.iterator();
		while (i.hasNext()) {
			out.println("<gem>");

			Gem g = (Gem) i.next();
			out.println("<name>" + g.getName() + "</name>");
			out.println("<location>" + g.getLocation().toString()
					+ "</location>");
			if (g.equals(fSelectedGem)) {
				out.println("<selected>true</selected>");
			}

			out.println("</gem>");
		}
		out.println("</gems>");
		out.flush();
	}
}
