/*******************************************************************************
 * Copyright (c) 2005 RadRails.org and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.radrails.org/legal/cpl-v10.html
 *******************************************************************************/
package org.radrails.rails.core.railsplugins;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.InputStreamReader;
import java.io.PrintWriter;
import java.io.Reader;
import java.io.StringReader;
import java.net.HttpURLConnection;
import java.net.URL;
import java.util.ArrayList;
import java.util.List;

import javax.xml.parsers.SAXParserFactory;

import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.NullProgressMonitor;
import org.radrails.rails.core.RailsLog;
import org.radrails.rails.internal.core.RailsPlugin;
import org.xml.sax.InputSource;
import org.xml.sax.XMLReader;

/**
 * Singleton class that manages access to a list of available Rails plugins. The
 * list is cached in the rails.ui plug-in state location, and can be refreshed
 * on-demand from the Rails Plugins web service.
 * 
 * @author mkent
 * 
 */
public class RailsPluginsManager {

	private static final String SERVICE_URL = "http://plugins.radrails.org/plugins";

	private static final String PLUGINS_FILE = "rails_plugins.xml";

	private static RailsPluginsManager fInstance;

	private List fPlugins;

	private RailsPluginsManager() {

	}

	public static RailsPluginsManager getInstance() {
		if (fInstance == null) {
			fInstance = new RailsPluginsManager();
		}
		return fInstance;
	}

	/**
	 * Gets a list of currently available Rails plugins. The list contains Maps,
	 * each of which holds information about a plugin in String to String
	 * mappings.
	 * 
	 * @return a list of plugins
	 * @throws Exception
	 */
	public List getPlugins() throws Exception {
		if (fPlugins == null) {
			fPlugins = new ArrayList();

			// Try to load from state file
			loadPlugins();

			// No results, try to update from web service
			if (fPlugins.isEmpty()) {
				updatePlugins(new NullProgressMonitor());
			}
		}
		return fPlugins;
	}

	/**
	 * Updates the current list of Rails plugins from the web service.
	 * 
	 * @throws Exception
	 *             if an error occurs
	 */
	public void updatePlugins(IProgressMonitor monitor) throws Exception {
		monitor.beginTask("Updating plugin list", 3);
		
		// Get latest list from web service
		monitor.subTask("Accessing plugin directory");
		URL url = new URL(SERVICE_URL);
		HttpURLConnection conn = (HttpURLConnection) url.openConnection();
		conn.setRequestMethod("GET");
		monitor.worked(1);
		
		monitor.subTask("Downloading plugin list");
		BufferedReader bufIn = new BufferedReader(new InputStreamReader(conn
				.getInputStream()));
		StringBuffer pxml = new StringBuffer();
		String l = null;
		while ((l = bufIn.readLine()) != null) {
			pxml.append(l);
		}
		monitor.worked(1);

		monitor.subTask("Processing plugin information");
		// Update the object list from the XML data
		StringReader strIn = new StringReader(pxml.toString());
		fPlugins = parsePluginsXML(strIn);

		// Cache XML data to file
		IPath loc = RailsPlugin.getInstance().getStateLocation().append(
				PLUGINS_FILE);
		File f = new File(loc.toOSString());
		PrintWriter fOut = new PrintWriter(
				new BufferedWriter(new FileWriter(f)));
		fOut.write(pxml.toString());
		fOut.flush();
		fOut.close();
		monitor.worked(1);
		monitor.done();
	}

	/**
	 * Loads the list of plugins from the state file.
	 */
	private void loadPlugins() {
		IPath loc = RailsPlugin.getInstance().getStateLocation().append(
				PLUGINS_FILE);
		File f = new File(loc.toOSString());
		if (f.exists()) {
			try {
				FileReader fis = new FileReader(f);
				fPlugins = parsePluginsXML(fis);
			} catch (Exception e) {
				RailsLog.logError("Error parsing Rails plugins XML", e);
			}
		}
	}

	/**
	 * Parses an XML file describing a list of Rails plugins and returns a List
	 * of Map objects, containing the plugin attributes mapped to their
	 * corresponding values.
	 * 
	 * @param rdr
	 *            the XML source
	 * @return a list of plugin Maps
	 * @throws Exception
	 *             if an error occurs
	 */
	private List parsePluginsXML(Reader rdr) throws Exception {
		XMLReader reader = SAXParserFactory.newInstance().newSAXParser()
				.getXMLReader();
		RailsPluginsContentHandler handler = new RailsPluginsContentHandler();
		reader.setContentHandler(handler);
		reader.parse(new InputSource(rdr));

		return handler.getRailsPlugins();
	}
}
