/*******************************************************************************
 * Copyright (c) 2005 RadRails.org and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.radrails.org/legal/cpl-v10.html
 *******************************************************************************/
package org.radrails.rails.core.yml;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.Reader;

/**
 * A basic implementation of YMLParser for use with Rails. The only elements
 * this parser will discover are nodes with contained mappings. This format is
 * found in the database.yml configuration file and test fixture files.
 * Malformed and unknown elements will generate exceptions.
 * 
 * @author mkent
 * 
 */
class SimpleYMLParser implements YMLParser {

	private static final char SEPARATOR = ':';

	private static final String COMMENT = "#";

	private YMLContentHandler handler;

	private boolean inNode;

	private boolean nodeAliased;

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.radrails.rails.core.yml.YMLParser#setContentHandler(org.radrails.rails.core.yml.YMLContentHandler)
	 */
	public void setContentHandler(YMLContentHandler handler) {
		this.handler = handler;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.radrails.rails.core.yml.YMLParser#parse(java.io.Reader)
	 */
	public void parse(Reader input) throws IOException, YMLParserException {
		if (handler == null) {
			fatal("No content handler set");
		}
		handler.startDocument();
		parseDocument(input);
		handler.endDocument();
	}

	/**
	 * Parses the YML document line by line.
	 * 
	 * @param input
	 * @throws IOException
	 * @throws YMLParserException
	 */
	private void parseDocument(Reader input) throws IOException,
			YMLParserException {
		inNode = false;
		nodeAliased = false;

		BufferedReader bufInput = new BufferedReader(input);
		String line = null;
		while ((line = bufInput.readLine()) != null) {
			// ignore comments and blank lines
			if (!line.trim().startsWith(COMMENT) && !line.trim().equals("")) {
				parseLine(line);
			}
		}
		bufInput.close();
	}

	/**
	 * Parses a single line from the YML document. Valid elements are node,
	 * mapping or alias. Other elements will generate an exception.
	 * 
	 * @param line
	 * @throws YMLParserException
	 */
	private void parseLine(String line) throws YMLParserException {
		// Node
		if (matchesNode(line)) {
			inNode = true;
			nodeAliased = false;

			int i = line.indexOf(SEPARATOR);

			// Report the node name to the handler
			String nodeName = line.substring(0, i);
			handler.node(nodeName);

			// Check for an alias
			String aliasName = line.substring(i + 1);
			if (aliasName.length() > 0) {
				nodeAliased = true;
				inNode = false;
				handler.alias(aliasName.trim());
			}
		}
		// Mapping
		else if (inNode && matchesMapping(line)) {
			line = line.trim();
			int i = line.indexOf(SEPARATOR);

			// Report the key and value of the mapping to the handler
			String key = line.substring(0, i);
			String value = line.substring(i + 1);
			handler.mapping(key.trim(), value.trim());
		}
		// Alias
		else if (inNode && !nodeAliased && matchesAlias(line)) {
			nodeAliased = true;
			inNode = false;
			handler.alias(line.trim());
		} else {
			fatalFormat(line);
		}
	}

	private boolean matchesNode(String str) {
		return str.matches("^\\S+\\s*:\\s*\\S*\\s*");
	}

	private boolean matchesAlias(String str) {
		return str.matches("^\\s{2}[\\S&&[^:]]+\\s*");
	}

	private boolean matchesMapping(String str) {
		return str.matches("^\\s{2}\\S+\\s*:\\s*\\S*\\s*");
	}

	private void fatalFormat(String text) throws YMLParserException {
		fatal("Invalid YML syntax: " + text);
	}

	private void fatal(String message) throws YMLParserException {
		fatal(message, null);
	}

	private void fatal(String message, Throwable t) throws YMLParserException {
		YMLParserException e = new YMLParserException(message, t);
		throw e;
	}

}
