/*******************************************************************************
 * Copyright (c) 2005 RadRails.org and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.radrails.org/legal/cpl-v10.html
 *******************************************************************************/

package org.radrails.rails.internal.core;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStreamReader;
import java.net.URL;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Observer;

import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.Plugin;
import org.eclipse.debug.core.DebugEvent;
import org.eclipse.debug.core.DebugPlugin;
import org.eclipse.debug.core.IDebugEventSetListener;
import org.eclipse.debug.core.model.IProcess;
import org.osgi.framework.BundleContext;
import org.radrails.internal.core.launching.IRailsLaunchConfigAttributes;
import org.radrails.rails.core.IRailsConstants;
import org.radrails.rails.core.RailsLog;
import org.rubypeople.rdt.core.RubyCore;

/**
 * Plugin class for the rails core plugin.
 * 
 * @author	mkent
 * 
 * @version	0.3.1
 */
public class RailsPlugin extends Plugin implements IDebugEventSetListener {

	private static RailsPlugin instance;
	private IProject currentlySelectedProject;
	private List projectObservers;

	/**
	 * Constructor.
	 */
	public RailsPlugin() {
		super();
		instance = this;
		currentlySelectedProject = null;
		projectObservers = new ArrayList();
		
	}

	public void start(BundleContext context) throws Exception {
		super.start(context);
		
		DebugPlugin.getDefault().addDebugEventListener(this);
	}

	public void stop(BundleContext context) throws Exception {
		super.stop(context);
		
		DebugPlugin.getDefault().addDebugEventListener(this);
		savePluginPreferences();
	}

	/**
	 * @return the singleton instance of this class
	 */
	public static RailsPlugin getInstance() {
		return instance;
	}
	
	public void setSelectedProject( IProject currentSelectedProject ) {
		this.currentlySelectedProject = currentSelectedProject;
		notifyObservers();
	}
	
	public IProject getSelectedProject() {
		return currentlySelectedProject;
	}
	
	
	private IProject getSelectedByNatureIDt( String natureId ) {

		try {
			if( ( currentlySelectedProject != null ) &&  currentlySelectedProject.hasNature( natureId )  ) {
				// Selected project nature matches
				return currentlySelectedProject;
			}
		} catch (CoreException e) {
			RailsLog.logError( "CoreException getting project nature for project: "
					+ currentlySelectedProject.getName(), e );
		}

		// Project does not have desired nature.
		return null;
	}
	

	public IProject getSelectedRubyProject() {
		return getSelectedByNatureIDt( RubyCore.NATURE_ID );
	}
		
	

	public IProject getSelectedRailsProject() {
		return getSelectedByNatureIDt( IRailsConstants.RAILS_PROJECT_NATURE );
	}
	

	public IProject getSelectedRailsOrRubyProject() {
		
		IProject retVal;
		
		if( (retVal = getSelectedRailsProject() ) == null ) {
			retVal = getSelectedRubyProject();
		}

		return retVal;
	}
	
	
	public void addProjectObserver( Observer observer ) {
		projectObservers.add( observer );
		observer.update(null, currentlySelectedProject);
	}
	
	public void deleteProjectObserver( Observer observer ) {
		projectObservers.remove( observer );
	}
	
	private void notifyObservers() {
		
		Iterator observerIt = projectObservers.iterator();
		while( observerIt.hasNext() ) {
			Observer obs = (Observer) observerIt.next();
			obs.update(  null, currentlySelectedProject );
		}
	}
	
	/**
	 * Writes a script from plugin jar to plugin metadata folder within the
	 * workspace.
	 * 
	 * @param rubyFile -
	 *            The file to place on the filesystem
	 * @return Absolute path to specified script file
	 */
	public String getRubyScriptPath(String rubyFile) {
		String directoryFile = getStateLocation()
				.toOSString()
				+ File.separator + rubyFile;
		File pluginDirFile = new File(directoryFile);

		if (!pluginDirFile.exists()) {
			try {
				pluginDirFile.createNewFile();
				URL u = getBundle().getEntry(
						"/ruby/" + rubyFile);
				BufferedReader input = new BufferedReader(
						new InputStreamReader(u.openStream()));
				FileWriter output = new FileWriter(pluginDirFile);
				String line;
				while ((line = input.readLine()) != null) {
					output.write(line);
					output.write('\n');
				}
				output.flush();
				output.close();
				input.close();
			} catch (IOException e) {
				RailsLog.logError("Error writing plugin script to metadata",e);
			}
		}

		String path = "";
		try {
			path = pluginDirFile.getCanonicalPath();
		} catch (IOException e) {
			RailsLog.logError("Error getting file path", e);
		}
		return path;
	}
	
	public String getRailsPath() {
		return getPluginPreferences().getString(IRailsConstants.PREF_RAILS_PATH);
	}
	
	public String getRakePath() {
		return getPluginPreferences().getString(IRailsConstants.PREF_RAKE_PATH);
	}
	
	public String getMongrelPath() {
		return getPluginPreferences().getString(IRailsConstants.PREF_MONGREL_PATH);
	}

	public void handleDebugEvents(DebugEvent[] events) {
		for(int i = 0; i < events.length; i++) {
			
			// Watch for termination of processes that modify the filesystem
			//  and refresh the modified project
			int kind = events[i].getKind();
			Object source = events[i].getSource();
			if((kind == DebugEvent.TERMINATE) && (source instanceof IProcess)) {
				IProcess iProc = (IProcess) source;
				String refresh = iProc.getAttribute(IRailsLaunchConfigAttributes.REQUIRES_REFRESH);
				if((refresh != null) && refresh.equals("true")) {
					String projName = iProc.getAttribute(IRailsLaunchConfigAttributes.PROJECT_NAME);
					IProject proj = ResourcesPlugin.getWorkspace().getRoot().getProject(projName);
					try {
						proj.refreshLocal(IResource.DEPTH_INFINITE, null);
					} catch (CoreException e) {
						RailsLog.logError("Error refreshing project", e);
					}
				}
			}
		}
	}
	

} // RailsPlugin