/*******************************************************************************
 * Copyright (c) 2005 RadRails.org and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.radrails.org/legal/cpl-v10.html
 *******************************************************************************/


package org.radrails.rails.internal.generators;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Observable;
import java.util.Observer;

import org.eclipse.core.resources.IProject;
import org.radrails.rails.core.RailsLog;
import org.radrails.rails.internal.core.RailsPlugin;

/**
 * Manages locating generators in the system rails 
 * install and the user's home directory (Environment)
 * and generators in rails projects.
 * 
 * @author andy
 *
 */
public class GeneratorLocatorsManager extends Observable implements Observer  {
	
	private static GeneratorLocatorsManager instance = null;
	private EnvironmentGeneratorLocator environmentGeneratorLocator;
	private static Map projectGeneratorLocators;
	
	private static IProject currentProject = null;
	
	/**
	 * Default constructor.
	 */
	private GeneratorLocatorsManager() {

		projectGeneratorLocators = new HashMap();
		RailsPlugin.getInstance().addProjectObserver( this );
		
		environmentGeneratorLocator = new EnvironmentGeneratorLocator();
		environmentGeneratorLocator.locateGenerators();
	}
	
	private IProject getCurrentProject() {
		
		if( currentProject == null ) {
			currentProject = RailsPlugin.getInstance().getSelectedProject();

			if( currentProject != null ) {
				projectGeneratorLocators.put( currentProject, new ProjectGeneratorsLocator(currentProject) );
			}
		}
		
		return currentProject;
	}
	
	/**
	 * @return the singleton instance of <code></code>.
	 */
	public static GeneratorLocatorsManager getInstance() {
		if (instance == null) {
			instance = new GeneratorLocatorsManager();
		}

		return instance;
	}

	
	/**
	 * Gets all the generators associated with the environment and the currently selected project,
	 * without actually doing any reparsing.
	 * 
	 * @return List of previously discovered generators in the environment and current project.
	 */
	public List getAllGenerators() {
		
		List retList = new ArrayList();
		retList.addAll( environmentGeneratorLocator.getLocatedGenerators() );
		
		IProject currentProject = getCurrentProject();
		
		if( currentProject !=null ) {
			GeneratorsLocator currentProjectLocator = ((GeneratorsLocator) projectGeneratorLocators.get( currentProject) );
			retList.addAll( currentProjectLocator.getLocatedGenerators() );
		}

		return retList;
	}
	
	public synchronized List refreshAllGenerators() {
		
		IProject currentProject = getCurrentProject();
		environmentGeneratorLocator.locateGenerators();
		
		if( currentProject != null ) {
			((GeneratorsLocator) projectGeneratorLocators.get(  currentProject) ).locateGenerators();
		} else {
			RailsLog.logError("Refresh Generators: current project is null", null);
		}
		
		return getAllGenerators();
		
	}


	public void update(Observable o, Object arg) {
		
		// Try to fetch a rails project, ignore the input argument.
		IProject inProject = RailsPlugin.getInstance().getSelectedRailsOrRubyProject();

		if( (inProject != null) && (currentProject !=  inProject)  ) {
			currentProject = inProject;
			
			// Search  rails project for generators.
			if(  !projectGeneratorLocators.containsKey( inProject ) ) {
				GeneratorsLocator locator = new ProjectGeneratorsLocator(inProject);
				locator.locateGenerators();
				projectGeneratorLocators.put( inProject, locator );
			}
		}
		
		setChanged();
		notifyObservers( inProject  );
		
	}

	public synchronized void addObserver(Observer arg0) {
		super.addObserver(arg0);
		arg0.update( this, currentProject );
	}
	
	

	
}
