/*******************************************************************************
 * Copyright (c) 2006 RadRails.org and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.radrails.org/legal/cpl-v10.html
 *******************************************************************************/

package org.radrails.rails.internal.ui.autotest;

import java.io.File;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.IResourceChangeEvent;
import org.eclipse.core.resources.IResourceChangeListener;
import org.eclipse.core.resources.IResourceDelta;
import org.eclipse.core.resources.IResourceDeltaVisitor;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.core.runtime.jobs.Job;
import org.eclipse.debug.core.DebugPlugin;
import org.eclipse.debug.core.ILaunch;
import org.eclipse.debug.core.ILaunchManager;
import org.eclipse.debug.core.IStreamListener;
import org.eclipse.debug.core.Launch;
import org.eclipse.debug.core.model.IProcess;
import org.eclipse.debug.core.model.IStreamMonitor;
import org.eclipse.jface.util.IPropertyChangeListener;
import org.eclipse.jface.util.PropertyChangeEvent;
import org.eclipse.ui.PlatformUI;
import org.radrails.rails.internal.core.RailsPlugin;
import org.radrails.rails.internal.core.RailsRuntime;
import org.radrails.rails.internal.ui.RailsUILog;

/**
 * Listens for resource change events. When Ruby files are saved, launches
 * associated Test::Unit suites based on preferences. For example, launches
 * corresponding unit test for models, functional tests for controllers, and so
 * forth. Exactly which tests are run for which classes is configurable via
 * preferences.
 * 
 * @author mkent
 * 
 */
public class AutotestManager implements IResourceChangeListener,
		IPropertyChangeListener {

	private Collection fRunListeners;
	
	private AutotestThread fAutotestThread;

	public AutotestManager() {
		fRunListeners = new ArrayList();
		if(AutotestHelper.runAutotestOnInterval()) {
			fAutotestThread = new AutotestThread();
			fAutotestThread.start();
		}
	}

	public void resourceChanged(IResourceChangeEvent event) {
		boolean runOnSave = AutotestHelper.runAutotestOnSave();
		try {
			if (runOnSave) {
				event.getDelta().accept(fDeltaVisitor);
			}
		} catch (CoreException e) {
			RailsUILog.logError("Error launching tests", e);
		}
	}

	private IResourceDeltaVisitor fDeltaVisitor = new IResourceDeltaVisitor() {
		public boolean visit(IResourceDelta delta) throws CoreException {
			switch (delta.getKind()) {
			case IResourceDelta.ADDED:
				// handle added resource
				break;
			case IResourceDelta.REMOVED:
				// handle removed resource
				break;
			case IResourceDelta.CHANGED:
				// only fire on content change events on the active editor file
				if((delta.getFlags() & IResourceDelta.CONTENT) == IResourceDelta.CONTENT) {
					if(delta.getResource() != null && delta.getResource() instanceof IFile) {
						IFile changedFile = (IFile) delta.getResource();
						IFile activeEditorFile = AutotestHelper.getActiveEditorFile();
						if(changedFile.getLocation().equals(activeEditorFile.getLocation())
								&& changedFile.getFileExtension().equals("rb")) {
							attemptTestLaunch(delta.getResource());
						}
					}
				}
				break;
			}
			return true;
		}
	};

	private void attemptTestLaunch(final IResource file) {
		Job j = new Job("Autotest suite") {
			protected IStatus run(IProgressMonitor monitor) {
				monitor.beginTask("Running autotest suite", 3);
				monitor.worked(1);

				System.out.println("gathering tests");
				final List tests = AutotestHelper.getTests(file.getProject(),
						file.getProjectRelativePath());
				monitor.worked(1);

				System.out.println("test suite");
				Iterator i = tests.iterator();
				while (i.hasNext()) {
					System.out.println("test=" + i.next());
				}

				if (!tests.isEmpty()) {
					System.out.println("launching tests");
					launchTests(file.getProject().getLocation().toFile(), tests);
				}
				monitor.worked(1);
				monitor.done();
				return Status.OK_STATUS;
			}
		};
		j.schedule();
	}

	private void launchTests(File workingDir, List testFiles) {
		List cmds = new ArrayList();

		// Load the autotest runner and put its directory on the load path
		String trPath = RailsPlugin.getInstance().getRubyScriptPath(
				"autotestrunner.rb");
		String trDirPath = trPath.substring(0, trPath
				.lastIndexOf(File.separator));
		cmds.add("-I" + trDirPath);

		// Load the autotest launcher
		String runFile = RailsPlugin.getInstance().getRubyScriptPath(
				"run_auto.rb");
		cmds.add(runFile);
		cmds.addAll(testFiles);

		try {
			Process proc = RailsRuntime.rubyExec((String[]) cmds
					.toArray(new String[] {}), workingDir);
			ILaunch launch = new Launch(null, ILaunchManager.RUN_MODE, null);
			IProcess iproc = DebugPlugin.newProcess(launch, proc, "autotest");

			IStreamMonitor stdOut = iproc.getStreamsProxy()
					.getOutputStreamMonitor();
			IStreamMonitor stdErr = iproc.getStreamsProxy()
					.getErrorStreamMonitor();

			stdOut.addListener(new IStreamListener() {

				public void streamAppended(final String text,
						IStreamMonitor monitor) {
					System.out.println("stdout: " + text);

					// runFinished(text);
				}

			});

			stdErr.addListener(new IStreamListener() {

				public void streamAppended(final String text,
						IStreamMonitor monitor) {
					System.out.println("stderr: " + text);
				}
			});

			proc.waitFor();
			runFinished(stdOut.getContents());
		} catch (CoreException e) {
			RailsUILog.logError("Error launching autotest suite", e);
		} catch (InterruptedException e) {
			RailsUILog.logError("Error waiting for autotest process", e);
		}
	}

	private void runFinished(final String text) {
		PlatformUI.getWorkbench().getDisplay().syncExec(new Runnable() {
			public void run() {
				Iterator i = fRunListeners.iterator();
				while (i.hasNext()) {
					IAutotestRunListener rl = (IAutotestRunListener) i.next();
					if (text.trim().startsWith("pass")) {
						rl.suitePass();
					} else if (text.trim().startsWith("fail")) {
						rl.suiteFail();
					} else {
						rl.suiteError();
					}
				}
			}
		});
	}

	public void addRunListener(IAutotestRunListener listener) {
		fRunListeners.add(listener);
	}

	public void removeRunListener(IAutotestRunListener listener) {
		fRunListeners.remove(listener);
	}

	public void propertyChange(PropertyChangeEvent event) {
		if (event.getProperty().equals(
				IAutotestPreferenceConstants.RUN_ON_INTERVAL)) {
			boolean enabled = ((Boolean) event.getNewValue()).booleanValue();
			if (enabled) {
				if(fAutotestThread != null) {
					fAutotestThread.stopRunning();
				}
				fAutotestThread = new AutotestThread();
				fAutotestThread.start();
			}
			else {
				fAutotestThread.stopRunning();
				fAutotestThread = null;
			}
		}

	}

	private class AutotestThread extends Thread {
		private boolean keepRunning;

		public AutotestThread() {
			keepRunning = true;
		}

		public void run() {
			while (keepRunning) {
				int intervalMinutes = AutotestHelper
						.getAutotestIntervalLength();
				long intervalMs = intervalMinutes * 60 * 1000;
				try {
					sleep(intervalMs);
				} catch (InterruptedException e) {
					RailsUILog.logError(
							"Error sleeping during autotest interval", e);
				}

				// Run autotest suite
				IFile activeEditorFile = AutotestHelper.getActiveEditorFile();
				if (activeEditorFile != null) {
					attemptTestLaunch(activeEditorFile);
				}
			}
		}

		public void stopRunning() {
			keepRunning = false;
		}
	}

}
