package org.radrails.rails.internal.ui.editors;

import java.util.HashMap;
import java.util.Map;

import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.jface.preference.PreferenceConverter;
import org.eclipse.jface.resource.DeviceResourceException;
import org.eclipse.jface.resource.JFaceResources;
import org.eclipse.jface.resource.LocalResourceManager;
import org.eclipse.jface.resource.StringConverter;
import org.eclipse.jface.text.TextAttribute;
import org.eclipse.jface.text.rules.IToken;
import org.eclipse.jface.text.rules.RuleBasedScanner;
import org.eclipse.jface.text.rules.Token;
import org.eclipse.jface.util.PropertyChangeEvent;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.RGB;
import org.eclipse.ui.PlatformUI;
import org.radrails.rails.internal.ui.RailsUILog;
import org.radrails.rails.internal.ui.preferences.IRailsPreferenceConstants;

/**
 * Same thing as AbstractManagedScanner (code literally copied and pasted)
 * except this class extends RuleBasedScanner instead of just ITokenScanner.
 * 
 * @author matt
 * 
 */
public abstract class AbstractManagedRuleBasedScanner extends RuleBasedScanner implements IAbstractManagedScanner {

	protected IPreferenceStore fPrefStore;
	
	protected Map fTokenMap;
	
	private LocalResourceManager fResourceManager;
	
	private String[] fColorKeys;
	private String[] fBoldKeys;
	private String[] fItalicKeys;
	
	protected AbstractManagedRuleBasedScanner() {
		fTokenMap = new HashMap();
		fResourceManager = new LocalResourceManager(JFaceResources.getResources());
		PlatformUI.getWorkbench().getDisplay().disposeExec(new Runnable() {
			public void run() {
				fResourceManager.dispose();
			}
		});
		fPrefStore = getPreferenceStore();
		initTokens();
	}
	
	protected abstract String[] getColorKeys();
	
	protected abstract IPreferenceStore getPreferenceStore();
	
	protected void initTokens() {
		fColorKeys = getColorKeys();
		fBoldKeys = new String[fColorKeys.length];
		fItalicKeys = new String[fColorKeys.length];
		
		for(int i = 0; i < fColorKeys.length; i++) {
			fBoldKeys[i] = fColorKeys[i] + IRailsPreferenceConstants.BOLD_SUFFIX;
			fItalicKeys[i] = fColorKeys[i] + IRailsPreferenceConstants.ITALIC_SUFFIX;
		}
		
		for(int i = 0; i < fColorKeys.length; i++) {
			TextAttribute textAtt = createTextAttribute(fColorKeys[i], fBoldKeys[i], fItalicKeys[i]);
			fTokenMap.put(fColorKeys[i], new Token(textAtt));
		}
	}

	protected IToken getToken(String key) {
		return (IToken) fTokenMap.get(key);
	}	
	
	private TextAttribute createTextAttribute(String colorKey, String boldKey, String italicKey) {	
		Color color = getColor(colorKey);

		int style = fPrefStore.getBoolean(boldKey) ? SWT.BOLD : SWT.NORMAL;
		style |= fPrefStore.getBoolean(italicKey) ? SWT.ITALIC : SWT.NORMAL;

		return new TextAttribute(color, null, style);
	}

	public boolean affectsBehavior(PropertyChangeEvent event) {
		return indexOf(event.getProperty()) >= 0;
	}

	public void adaptToPreferenceChange(PropertyChangeEvent event) {
		String p = event.getProperty();
		int index = indexOf(p);
		IToken token = getToken(fColorKeys[index]);
		if (fColorKeys[index].equals(p)) {
			adaptToColorChange(token, event);
		}
		else if (fBoldKeys[index].equals(p)) {
			adaptToStyleChange(token, event, SWT.BOLD);
		}
		else if (fItalicKeys[index].equals(p)) {
			adaptToStyleChange(token, event, SWT.ITALIC);
		}
	}

	private void adaptToStyleChange(IToken token, PropertyChangeEvent event, int styleAttribute) {
		boolean newValue = false;
		Object valueObj = event.getNewValue();
		if (valueObj instanceof Boolean) {
			newValue = ((Boolean) valueObj).booleanValue();
		}
		else if (IPreferenceStore.TRUE.equals(valueObj)) {
			newValue = true;
		}

		Object data = token.getData();
		if (data instanceof TextAttribute) {
			TextAttribute oldAttr = (TextAttribute) data;
			boolean oldValue = (oldAttr.getStyle() & styleAttribute) == styleAttribute;
			if (oldValue != newValue) {
				Token tok = (Token) token;
				tok.setData(new TextAttribute(oldAttr.getForeground(), oldAttr.getBackground(), newValue ? oldAttr.getStyle() | styleAttribute : oldAttr.getStyle() & ~styleAttribute));
			}
		}
	}

	private void adaptToColorChange(IToken token, PropertyChangeEvent event) {
		RGB rgb = null;
		Object value = event.getNewValue();
		if (value instanceof RGB)
			rgb = (RGB) value;
		else if (value instanceof String) {
			rgb = StringConverter.asRGB((String) value);
		}

		if (rgb != null) {
			String property = event.getProperty();
			Color color = getColor(property);

			Object data = token.getData();
			if (data instanceof TextAttribute) {
				TextAttribute oldAttr = (TextAttribute) data;
				Token tok = (Token) token;
				tok.setData(new TextAttribute(color, oldAttr.getBackground(), oldAttr.getStyle()));
			}
		}
	}

	private int indexOf(String property) {
		if (property != null) {
			int length = fColorKeys.length;
			for (int i = 0; i < length; i++) {
				if (property.equals(fColorKeys[i]) || property.equals(fBoldKeys[i]) || property.equals(fItalicKeys[i])) {
					return i;
				}
			}
		}
		return -1;
	}
	
	private Color getColor(String key) {
		Color color = null;
		RGB rgb = PreferenceConverter.getColor(fPrefStore, key);
		try {
			color = fResourceManager.createColor(rgb);
		} catch (DeviceResourceException e) {
			RailsUILog.logError("Unable to create color", e);
		}
		return color;
	}
}
