/*******************************************************************************
 * Copyright (c) 2005 RadRails.org and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.radrails.org/legal/cpl-v10.html
 *******************************************************************************/

package org.radrails.rails.internal.ui.editors;

import org.eclipse.jface.text.BadLocationException;
import org.eclipse.jface.text.IDocument;
import org.eclipse.jface.text.source.SourceViewer;
import org.eclipse.swt.events.VerifyEvent;
import org.eclipse.swt.graphics.Point;
import org.radrails.rails.internal.ui.RailsUILog;

/**
 * Closer for the following character pairs
 * <ul>
 * <li>[ ]</li>
 * <li>{ }</li>
 * <li>( )</li>
 * </ul>
 * 
 * @author mkent
 * 
 */
public class BracketCloser extends AbstractCloser {

	/**
	 * Constructor.
	 * 
	 * @param sourceViewer
	 *            the <code>SourceViewer</code> that the closer operates on
	 */
	public BracketCloser(SourceViewer sourceViewer) {
		super(sourceViewer);
	}

	public void verifyKey(VerifyEvent event) {
		if (!fEnabled) {
			return;
		}

		// Don't auto-close inside Ruby comments
		if (isCurrentLineRubyComment()) {
			return;
		}

		// Don't auto-close inside string literals
		if (isCursorInsideString()) {
			return;
		}

		IDocument document = fSourceViewer.getDocument();
		Point selection = fSourceViewer.getSelectedRange();
		int cursorOffset = selection.x;
		int selectionLength = selection.y;

		// If character is an open bracket, then insert the closing peer
		if ((event.character == '[') || (event.character == '{')
				|| (event.character == '(')) {
			try {
				StringBuffer sb = new StringBuffer();
				sb.append(event.character);
				sb.append(getPeer(event.character));

				document.replace(cursorOffset, selectionLength, sb.toString());

				event.doit = false;
				moveCursor(1);
			} catch (BadLocationException e) {
				RailsUILog.logError("Bad location", e);
			}
		}
		// If character is a close bracket, then overwrite if appropriate
		else if ((event.character == ']') || (event.character == '}')
				|| (event.character == ')')) {
			try {
				// Only overwrite close bracket is selection is empty
				if (selectionLength == 0) {
					char nextChar = document.getChar(cursorOffset);

					// If the next character is the same close bracket just
					// typed, skip the cursor over it
					if (nextChar == event.character) {
						event.doit = false;
						moveCursor(1);
					}
				}
			} catch (BadLocationException e) {
				RailsUILog.logError("Bad location", e);
			}
		}
		// If character is a backspace, then do a smart delete
		else if (event.character == BACKSPACE) {
			try {
				// Only do smart delete if selection is empty
				if (selectionLength == 0) {
					char prevChar = document.getChar(cursorOffset - 1);

					// If the previous character is an open bracket,
					// then check for a closing peer
					if ((prevChar == '[') || (prevChar == '{')
							|| (prevChar == '(')) {

						char nextChar = document.getChar(cursorOffset);

						// If the next character is the closing peer to the
						// previous, delete both
						if (nextChar == getPeer(prevChar)) {
							document.replace(cursorOffset - 1, 2, "");
							event.doit = false;
						}
					}
				}
			} catch (BadLocationException e) {
				RailsUILog.logError("Bad location", e);
			}
		}
	}

	/**
	 * Helper method to return the peer of a given character.
	 * 
	 * @param ch
	 *            a character
	 * @return the character's peer
	 */
	private char getPeer(char ch) {
		switch (ch) {
		case '(':
			return ')';

		case ')':
			return '(';

		case '{':
			return '}';

		case '}':
			return '{';

		case '[':
			return ']';

		case ']':
			return '[';

		default:
			return 0;
		}
	}

}
