/*******************************************************************************
 * Copyright (c) 2005 RadRails.org and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.radrails.org/legal/cpl-v10.html
 *******************************************************************************/

package org.radrails.rails.internal.ui.editors;

import org.eclipse.jface.text.BadLocationException;
import org.eclipse.jface.text.IDocument;
import org.eclipse.jface.text.source.SourceViewer;
import org.eclipse.swt.events.VerifyEvent;
import org.eclipse.swt.graphics.Point;
import org.radrails.rails.internal.ui.RailsUILog;

/**
 * Closer for <code>def</code>/<code>end</code> pairs and
 * <code>class</code>/<code>end</code> pairs.
 * 
 * @author mkent
 * 
 */
public class EndCloser extends AbstractCloser {

	/**
	 * Constructor.
	 * 
	 * @param sourceViewer
	 *            the <code>SourceViewer</code> that the closer operates on
	 */
	public EndCloser(SourceViewer sourceViewer) {
		super(sourceViewer);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.swt.custom.VerifyKeyListener#verifyKey(org.eclipse.swt.events.VerifyEvent)
	 */
	public void verifyKey(VerifyEvent event) {
		if (!fEnabled) {
			return;
		}

		// Don't auto-close inside Ruby comments
		if (isCurrentLineRubyComment()) {
			return;
		}

		// Don't auto-close inside string literals
		if (isCursorInsideString()) {
			return;
		}

		IDocument document = fSourceViewer.getDocument();
		Point selection = fSourceViewer.getSelectedRange();
		int cursorOffset = selection.x;
		int selectionLength = selection.y;

		// If character is enter, check for method or class definition
		if (event.character == CARRAIGE_RETURN) {
			// Only close if selection is empty
			if (selectionLength == 0) {
				try {
					int lineNumber = document.getLineOfOffset(cursorOffset);
					int lineLength = document.getLineLength(lineNumber);
					int lineOffset = document.getLineOffset(lineNumber);

					String lineText = document.get(lineOffset, lineLength);

					// If line starts with def or class, add a closing end
					if (lineText.trim().startsWith("def ")
							|| lineText.trim().startsWith("class ")) {
						int nextLineLength = document
								.getLineLength(lineNumber + 1);
						int nextLineOffset = document
								.getLineOffset(lineNumber + 1);
						String nextLineText = document.get(nextLineOffset,
								nextLineLength);

						// Check for one liners
						if (lineText.trim().endsWith("end")) {
							return;
						}
						
						// Check for an end on the next line
						if (nextLineText.trim().equals("end")) {
							return;
						}

						// Grab the current amount of indent space
						String indentSpace = null;
						if (lineText.trim().startsWith("def ")) {
							int indentLength = lineText.indexOf("def");
							indentSpace = lineText.substring(0, indentLength);
						} else if (lineText.trim().startsWith("class ")) {
							int indentLength = lineText.indexOf("class");
							indentSpace = lineText.substring(0, indentLength);
						}

						StringBuffer sb = new StringBuffer();
						sb.append(NEWLINE);
						sb.append(indentSpace);
						sb.append(TAB);

						// Save the cursor position in between the def and end
						int cursorDelta = sb.length();
						sb.append(NEWLINE);
						sb.append(indentSpace);
						sb.append("end");

						document.replace(cursorOffset, selectionLength, sb
								.toString());

						event.doit = false;
						moveCursor(cursorDelta);
					}
				} catch (BadLocationException e) {
					RailsUILog.logError("Bad location", e);
				}
			}
		}
	}

}
