/*******************************************************************************
 * Copyright (c) 2005 RadRails.org and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.radrails.org/legal/cpl-v10.html
 *******************************************************************************/

package org.radrails.rails.internal.ui.editors;

import org.eclipse.jface.text.BadLocationException;
import org.eclipse.jface.text.IDocument;
import org.eclipse.jface.text.source.SourceViewer;
import org.eclipse.swt.events.VerifyEvent;
import org.eclipse.swt.graphics.Point;
import org.radrails.rails.internal.ui.RailsUILog;

/**
 * Closer for single quotes and double quotes.
 * 
 * @author mkent
 * 
 */
public class QuoteCloser extends AbstractCloser {

	/**
	 * Constructor.
	 * 
	 * @param sourceViewer the <code>SourceViewer</code> that the closer operates on
	 */
	public QuoteCloser(SourceViewer sourceViewer) {
		super(sourceViewer);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.swt.custom.VerifyKeyListener#verifyKey(org.eclipse.swt.events.VerifyEvent)
	 */
	public void verifyKey(VerifyEvent event) {
		if (!fEnabled) {
			return;
		}
		
		// Don't auto-close inside Ruby comments
		if(isCurrentLineRubyComment()) {
			return;
		}
		
		IDocument document = fSourceViewer.getDocument();
		Point selection = fSourceViewer.getSelectedRange();
		int cursorOffset = selection.x;
		int selectionLength = selection.y;

		// If character is a quote, insert closing peer or overwrite
		if ((event.character == '"') || (event.character == '\'')) {
			try {
				char prevChar = document.getChar(cursorOffset - 1);

				// If the quote was escaped, ignore it
				if (prevChar == '\\') {
					return;
				}

				char nextChar = document.getChar(cursorOffset);

				// If the next character is the same quote just typed, skip the
				// cursor over it
				if (nextChar == event.character) {
					// Only skip over if selection is empty
					if (selectionLength == 0) {
						event.doit = false;
						moveCursor(1);
					}
				}
				// Otherwise, insert the closing peer quote
				else {
					// Don't auto-close quotes inside string literals
					if(isCursorInsideString()) {
						return;
					}
					
					StringBuffer sb = new StringBuffer();
					sb.append(event.character);
					sb.append(getPeer(event.character));

					document.replace(cursorOffset, selectionLength, sb
							.toString());

					event.doit = false;
					moveCursor(1);
				}
			} catch (BadLocationException e) {
				RailsUILog.logError("Bad location", e);
			}
		}
		// If character is a backspace, then do a smart delete
		else if (event.character == BACKSPACE) {
			try {
				// Only do smart delete if selection is empty
				if (selectionLength == 0) {
					char prevChar = document.getChar(cursorOffset - 1);

					// If the previous character is an open bracket or quote,
					// then check for a closing peer
					if ((prevChar == '[') || (prevChar == '{')
							|| (prevChar == '(') || (prevChar == '"')
							|| (prevChar == '\'')) {

						char nextChar = document.getChar(cursorOffset);

						// If the next character is the closing peer to the
						// previous, delete both
						if (nextChar == getPeer(prevChar)) {
							document.replace(cursorOffset - 1, 2, "");
							event.doit = false;
						}
					}
				}
			} catch (BadLocationException e) {
				RailsUILog.logError("Bad location", e);
			}
		}
	}

	/**
	 * Helper method to return the peer of a given character.
	 * 
	 * @param ch
	 *            a character
	 * @return the character's peer
	 */
	private char getPeer(char ch) {
		switch (ch) {

		case '"':
			return '"';

		case '\'':
			return '\'';

		default:
			return 0;
		}
	}

}
