/*******************************************************************************
 * Copyright (c) 2005 RadRails.org and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.radrails.org/legal/cpl-v10.html
 *******************************************************************************/

package org.radrails.rails.internal.ui.editors.css;

import java.util.ResourceBundle;

import org.eclipse.jface.action.IAction;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.jface.text.BadLocationException;
import org.eclipse.jface.text.IDocument;
import org.eclipse.jface.text.source.ISourceViewer;
import org.eclipse.jface.text.source.IVerticalRuler;
import org.eclipse.jface.util.PropertyChangeEvent;
import org.eclipse.swt.custom.StyledText;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.ui.editors.text.EditorsUI;
import org.eclipse.ui.editors.text.ITextEditorHelpContextIds;
import org.eclipse.ui.editors.text.TextEditor;
import org.eclipse.ui.ide.IDEActionFactory;
import org.eclipse.ui.texteditor.AddTaskAction;
import org.eclipse.ui.texteditor.ChainedPreferenceStore;
import org.eclipse.ui.texteditor.ITextEditorActionDefinitionIds;
import org.eclipse.ui.texteditor.ResourceAction;
import org.eclipse.ui.texteditor.TextOperationAction;
import org.radrails.rails.internal.ui.RailsUIPlugin;
import org.radrails.rails.internal.ui.editors.IManagedSourceViewerConfiguration;

/**
 * Editor for CSS documents.
 * 
 * @author	mbaumbach
 * 
 * @version	0.4.1
 */
public class CSSEditor extends TextEditor {

	public static final String CSS_PARTITIONING = "__css_partitioning";

	/**
	 * @see org.eclipse.ui.texteditor.AbstractDecoratedTextEditor#initializeEditor()
	 */
	protected void initializeEditor() {
		super.initializeEditor();

		// Add the rails ui plugin preferences to the editor
		IPreferenceStore pluginStore = RailsUIPlugin.getInstance()
				.getPreferenceStore();
		IPreferenceStore eclipseUIStore = EditorsUI.getPreferenceStore();
		setPreferenceStore(new ChainedPreferenceStore(new IPreferenceStore[] {
				pluginStore, eclipseUIStore }));

		setSourceViewerConfiguration(new CSSEditorSourceViewerConfiguration());
	}

	/**
	 * @see org.eclipse.ui.texteditor.AbstractTextEditor#affectsTextPresentation(org.eclipse.jface.util.PropertyChangeEvent)
	 */
	protected boolean affectsTextPresentation(PropertyChangeEvent event) {
		return ((IManagedSourceViewerConfiguration) getSourceViewerConfiguration()).affectsTextPresentation(event) ||
			super.affectsTextPresentation(event);
	}

	protected void handlePreferenceStoreChanged(PropertyChangeEvent event) {
		((IManagedSourceViewerConfiguration) getSourceViewerConfiguration()).handlePropertyChangeEvent(event);
		super.handlePreferenceStoreChanged(event);
	}

	protected ISourceViewer createSourceViewer(Composite parent, IVerticalRuler ruler, int styles) {
		fAnnotationAccess = createAnnotationAccess();
		fOverviewRuler = createOverviewRuler(getSharedColors());

		ISourceViewer viewer = new CSSSourceViewer(parent, ruler, getOverviewRuler(), isOverviewRulerVisible(), styles);
		// ensure decoration support has been created and configured.
		getSourceViewerDecorationSupport(viewer);
		return viewer;
	}
	
	/**
	 * @see org.eclipse.ui.texteditor.AbstractTextEditor#createActions()
	 */
	protected void createActions() {
		super.createActions();

		ResourceBundle[] bundle = { CSSEditorMessages.getResourceBundle(), HTMLEditorMessages.getResourceBundle() };
		for( int i = 0; i < bundle.length; i++ ) {
			IAction a = new TextOperationAction(bundle[i], "ContentAssistProposal.",
					this, ISourceViewer.CONTENTASSIST_PROPOSALS);
	
			// CTRL+Space key doesn't work without making this call
			a
					.setActionDefinitionId(ITextEditorActionDefinitionIds.CONTENT_ASSIST_PROPOSALS);
			setAction("ContentAssistProposal", a);
			a = new TextOperationAction(bundle[i], "ContentAssistTip.", this,
					ISourceViewer.CONTENTASSIST_CONTEXT_INFORMATION);
			a
					.setActionDefinitionId(ITextEditorActionDefinitionIds.CONTENT_ASSIST_CONTEXT_INFORMATION);
			setAction("ContentAssistTip", a);
	
			a = new TextOperationAction(bundle[i], "ContentFormatProposal.", this,
					ISourceViewer.FORMAT);
			setAction("ContentFormatProposal", a);
	
			// Add the task action to the Edit pulldown menu (bookmark action is
			// 'free')
			ResourceAction ra = new AddTaskAction(bundle[i], "AddTask.", this);
			ra.setHelpContextId(ITextEditorHelpContextIds.ADD_TASK_ACTION);
			ra.setActionDefinitionId(ITextEditorActionDefinitionIds.ADD_TASK);
			setAction(IDEActionFactory.ADD_TASK.getId(), ra);
			
			a = new CSSFormatAction(bundle[i], "Format.", this);
			a.setActionDefinitionId("org.radrails.rails.ui.editors.format");
			setAction("Format", a);
		}
	}
	
	public void editorContextMenuAboutToShow(IMenuManager menu) {
		super.editorContextMenuAboutToShow(menu);
		addAction(menu, "ContentAssistProposal");
		addAction(menu, "ContentAssistTip");
		addAction(menu, "Format");
	}
	
	public void setCaretPosition(CaretPosition pos) {

		try {
			int lineOffset = this.getSourceViewer().getDocument()
					.getLineOffset(pos.line);
			this.selectAndReveal(lineOffset + pos.column, 0);
		} catch (BadLocationException e) {
		}
	}

	public class CaretPosition {

		protected CaretPosition(int line, int column) {
			this.line = line;
			this.column = column;
		}

		protected int getColumn() {
			return column;
		}

		protected int getLine() {
			return line;
		}

		private int line;

		private int column;

	}

	public CaretPosition getCaretPosition() {
		// needed for positioning the cursor after formatting without selection

		StyledText styledText = this.getSourceViewer().getTextWidget();
		int caret = widgetOffset2ModelOffset(getSourceViewer(), styledText
				.getCaretOffset());
		IDocument document = getSourceViewer().getDocument();
		try {
			int line = document.getLineOfOffset(caret);
			int lineOffset = document.getLineOffset(line);
			return new CaretPosition(line, caret - lineOffset);
		} catch (BadLocationException e) {
			return new CaretPosition(0, 0);
		}
	}
	
} // CSSEditor