/*******************************************************************************
 * Copyright (c) 2005 RadRails.org and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.radrails.org/legal/cpl-v10.html
 *******************************************************************************/

package org.radrails.rails.internal.ui.editors.css;

import org.eclipse.jface.text.IDocument;
import org.eclipse.jface.text.contentassist.ContentAssistant;
import org.eclipse.jface.text.contentassist.IContentAssistant;
import org.eclipse.jface.text.presentation.IPresentationReconciler;
import org.eclipse.jface.text.presentation.PresentationReconciler;
import org.eclipse.jface.text.rules.DefaultDamagerRepairer;
import org.eclipse.jface.text.source.ISourceViewer;
import org.eclipse.jface.text.source.SourceViewerConfiguration;
import org.eclipse.jface.util.PropertyChangeEvent;
import org.eclipse.swt.graphics.RGB;
import org.radrails.rails.internal.ui.RailsUIColorProvider;
import org.radrails.rails.internal.ui.RailsUIPlugin;
import org.radrails.rails.internal.ui.editors.IAbstractManagedScanner;
import org.radrails.rails.internal.ui.editors.IManagedSourceViewerConfiguration;
import org.radrails.rails.internal.ui.preferences.IRailsPreferenceConstants;

/**
 * The source viewer configuration for the CSS editor.
 * 
 * @author mbaumbach
 * 
 * @version 0.4.1
 */
public class CSSEditorSourceViewerConfiguration extends
		SourceViewerConfiguration implements IManagedSourceViewerConfiguration {

	private IAbstractManagedScanner fSelectorScanner;
	private IAbstractManagedScanner fCommentScanner;
	private IAbstractManagedScanner fCodeScanner;
	
	public CSSEditorSourceViewerConfiguration() {
		initScanners();
	}
	
	private void initScanners() {
		fSelectorScanner = new CSSSelectorScanner();
		fCommentScanner = new CSSCommentScanner();
		fCodeScanner = new CSSCodeScanner();
	}
	
	/**
	 * @see org.eclipse.jface.text.source.SourceViewerConfiguration#getConfiguredDocumentPartitioning(org.eclipse.jface.text.source.ISourceViewer)
	 */
	public String getConfiguredDocumentPartitioning(ISourceViewer sourceViewer) {
		return CSSEditor.CSS_PARTITIONING;
	}

	/**
	 * @see org.eclipse.jface.text.source.SourceViewerConfiguration#getConfiguredContentTypes(org.eclipse.jface.text.source.ISourceViewer)
	 */
	public String[] getConfiguredContentTypes(ISourceViewer sourceViewer) {
		return new String[] { IDocument.DEFAULT_CONTENT_TYPE,
				CSSPartitionScanner.CSS_PROPERTIES,
				CSSPartitionScanner.CSS_COMMENTS };
	}

	/**
	 * @see org.eclipse.jface.text.source.SourceViewerConfiguration#getPresentationReconciler(org.eclipse.jface.text.source.ISourceViewer)
	 */
	public IPresentationReconciler getPresentationReconciler(
			ISourceViewer sourceViewer) {

		PresentationReconciler reconciler = new PresentationReconciler();
		reconciler
				.setDocumentPartitioning(getConfiguredDocumentPartitioning(sourceViewer));

		DefaultDamagerRepairer dr = new DefaultDamagerRepairer(
				fSelectorScanner);
		reconciler.setDamager(dr, IDocument.DEFAULT_CONTENT_TYPE);
		reconciler.setRepairer(dr, IDocument.DEFAULT_CONTENT_TYPE);

		dr = new DefaultDamagerRepairer(fCodeScanner);
		reconciler.setDamager(dr, CSSPartitionScanner.CSS_PROPERTIES);
		reconciler.setRepairer(dr, CSSPartitionScanner.CSS_PROPERTIES);

		dr = new DefaultDamagerRepairer(fCommentScanner);
		reconciler.setDamager(dr, CSSPartitionScanner.CSS_COMMENTS);
		reconciler.setRepairer(dr, CSSPartitionScanner.CSS_COMMENTS);

		return reconciler;
	}

	/**
	 * @see org.eclipse.jface.text.source.SourceViewerConfiguration#getContentAssistant(org.eclipse.jface.text.source.ISourceViewer)
	 */
	public IContentAssistant getContentAssistant(ISourceViewer sourceViewer) {

		ContentAssistant assistant = new ContentAssistant();
		assistant
				.setDocumentPartitioning(getConfiguredDocumentPartitioning(sourceViewer));
		assistant.setContentAssistProcessor(new HTMLCompletionProcessor(),
				IDocument.DEFAULT_CONTENT_TYPE);
		assistant.setContentAssistProcessor(new CSSCompletionProcessor(),
				CSSPartitionScanner.CSS_PROPERTIES);
		assistant.enableAutoInsert(true);

		assistant.setAutoActivationDelay(200);
		assistant
				.setProposalPopupOrientation(IContentAssistant.CONTEXT_INFO_BELOW);
		assistant
				.setContextInformationPopupOrientation(IContentAssistant.CONTEXT_INFO_ABOVE);
		assistant.setContextInformationPopupBackground(RailsUIColorProvider
				.getInstance().getColor(new RGB(150, 150, 0)));

		return assistant;
	}

	/**
	 * @see org.eclipse.jface.text.source.SourceViewerConfiguration#getTabWidth(org.eclipse.jface.text.source.ISourceViewer)
	 */
	public int getTabWidth(ISourceViewer sourceViewer) {
		return RailsUIPlugin.getInstance().getPreferenceStore().getInt(
				IRailsPreferenceConstants.CSS_INDENTATION);
	}

	public boolean affectsTextPresentation(PropertyChangeEvent event) {
		return fSelectorScanner.affectsBehavior(event) ||
			fCommentScanner.affectsBehavior(event) ||
			fCodeScanner.affectsBehavior(event);
	}

	public void handlePropertyChangeEvent(PropertyChangeEvent event) {
		if(fSelectorScanner.affectsBehavior(event)) {
			fSelectorScanner.adaptToPreferenceChange(event);
		}
		if(fCommentScanner.affectsBehavior(event)) {
			fCommentScanner.adaptToPreferenceChange(event);
		}
		if(fCodeScanner.affectsBehavior(event)) {
			fCodeScanner.adaptToPreferenceChange(event);
		}
	}

} // CSSSourceViewerConfiguration
