/*******************************************************************************
 * Copyright (c) 2005 RadRails.org and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.radrails.org/legal/cpl-v10.html
 *******************************************************************************/

package org.radrails.rails.internal.ui.editors.rhtml;

import org.eclipse.jface.text.IDocument;
import org.eclipse.jface.text.contentassist.ContentAssistant;
import org.eclipse.jface.text.contentassist.IContentAssistant;
import org.eclipse.jface.text.presentation.IPresentationReconciler;
import org.eclipse.jface.text.presentation.PresentationReconciler;
import org.eclipse.jface.text.rules.DefaultDamagerRepairer;
import org.eclipse.jface.text.source.ISourceViewer;
import org.eclipse.jface.text.source.SourceViewerConfiguration;
import org.eclipse.jface.util.PropertyChangeEvent;
import org.eclipse.swt.graphics.RGB;
import org.radrails.rails.internal.ui.RailsUIColorProvider;
import org.radrails.rails.internal.ui.RailsUIPlugin;
import org.radrails.rails.internal.ui.editors.IAbstractManagedScanner;
import org.radrails.rails.internal.ui.editors.IManagedSourceViewerConfiguration;
import org.radrails.rails.internal.ui.preferences.IRailsPreferenceConstants;

/**
 * RHTML Editor source viewer configuraiton
 * 
 * @author Kyle
 */
public class RHTMLEditorSourceViewerConfiguration extends
		SourceViewerConfiguration implements IManagedSourceViewerConfiguration {

	private IAbstractManagedScanner fDefScanner;
	private IAbstractManagedScanner fTagScanner;
	private IAbstractManagedScanner fCommentScanner;
	private IAbstractManagedScanner fDocTypeScanner;
	private IAbstractManagedScanner fExpScanner;
	
	public RHTMLEditorSourceViewerConfiguration() {
		initScanners();
	}
	
	private void initScanners() {
		fDefScanner = new DefaultScanner();
		fTagScanner = new RHTMLTagScanner();
		fCommentScanner = new HTMLCommentScanner();
		fDocTypeScanner = new HTMLDocTypeScanner();
		fExpScanner = new RubyExpressionScanner();
	}
	
	public boolean affectsTextPresentation(PropertyChangeEvent event) {
		return fDefScanner.affectsBehavior(event) ||
			fTagScanner.affectsBehavior(event) ||
			fCommentScanner.affectsBehavior(event) ||
			fDocTypeScanner.affectsBehavior(event) ||
			fExpScanner.affectsBehavior(event);
	}
  
	public void handlePropertyChangeEvent(PropertyChangeEvent event) {
		if(fDefScanner.affectsBehavior(event)) {
			fDefScanner.adaptToPreferenceChange(event);
		}
		if(fTagScanner.affectsBehavior(event)) {
			fTagScanner.adaptToPreferenceChange(event);
		}
		if(fCommentScanner.affectsBehavior(event)) {
			fCommentScanner.adaptToPreferenceChange(event);
		}
		if(fDocTypeScanner.affectsBehavior(event)) {
			fDocTypeScanner.adaptToPreferenceChange(event);
		}
		if(fExpScanner.affectsBehavior(event)) {
			fExpScanner.adaptToPreferenceChange(event);
		}
	}
	  
	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.text.source.SourceViewerConfiguration#getConfiguredContentTypes(org.eclipse.jface.text.source.ISourceViewer)
	 */
	public String[] getConfiguredContentTypes(ISourceViewer sourceViewer) {
		return new String[] { IDocument.DEFAULT_CONTENT_TYPE,
				RHTMLPartitionScanner.RHTML_TAG,
				RHTMLPartitionScanner.RHTML_MULTILINE_COMMENT,
				RHTMLPartitionScanner.RHTML_DOCTYPE,
				RHTMLPartitionScanner.RHTML_RUBY_EXPRESSION };
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.text.source.SourceViewerConfiguration#getConfiguredDocumentPartitioning(org.eclipse.jface.text.source.ISourceViewer)
	 */
	public String getConfiguredDocumentPartitioning(ISourceViewer sourceViewer) {
		return RailsUIPlugin.RHTML_PARTITIONING;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.text.source.SourceViewerConfiguration#getPresentationReconciler(org.eclipse.jface.text.source.ISourceViewer)
	 */
	public IPresentationReconciler getPresentationReconciler(
			ISourceViewer sourceViewer) {
		PresentationReconciler reconciler = new PresentationReconciler();
		reconciler
				.setDocumentPartitioning(getConfiguredDocumentPartitioning(sourceViewer));

		DefaultDamagerRepairer dr = new DefaultDamagerRepairer(fDefScanner);
		reconciler.setDamager(dr, IDocument.DEFAULT_CONTENT_TYPE);
		reconciler.setRepairer(dr, IDocument.DEFAULT_CONTENT_TYPE);

		dr = new DefaultDamagerRepairer(fTagScanner);
		reconciler.setDamager(dr, RHTMLPartitionScanner.RHTML_TAG);
		reconciler.setRepairer(dr, RHTMLPartitionScanner.RHTML_TAG);

		dr = new DefaultDamagerRepairer(fCommentScanner);
		reconciler
				.setDamager(dr, RHTMLPartitionScanner.RHTML_MULTILINE_COMMENT);
		reconciler.setRepairer(dr,
				RHTMLPartitionScanner.RHTML_MULTILINE_COMMENT);

		dr = new DefaultDamagerRepairer(fDocTypeScanner);
		reconciler.setDamager(dr, RHTMLPartitionScanner.RHTML_DOCTYPE);
		reconciler.setRepairer(dr, RHTMLPartitionScanner.RHTML_DOCTYPE);

		dr = new DefaultDamagerRepairer(fExpScanner);
		reconciler.setDamager(dr, RHTMLPartitionScanner.RHTML_RUBY_EXPRESSION);
		reconciler.setRepairer(dr, RHTMLPartitionScanner.RHTML_RUBY_EXPRESSION);

		return reconciler;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.text.source.SourceViewerConfiguration#getContentAssistant(org.eclipse.jface.text.source.ISourceViewer) /*
	 *      (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.text.source.SourceViewerConfiguration#getContentAssistant(org.eclipse.jface.text.source.ISourceViewer)
	 */
	public IContentAssistant getContentAssistant(ISourceViewer sourceViewer) {
		ContentAssistant assistant = new ContentAssistant();
		assistant
				.setDocumentPartitioning(getConfiguredDocumentPartitioning(sourceViewer));
		RHTMLCompletionProcessor rhtmlCp = new RHTMLCompletionProcessor();

		assistant.setContentAssistProcessor(rhtmlCp,
				IDocument.DEFAULT_CONTENT_TYPE);
		assistant.setContentAssistProcessor(rhtmlCp,
				RHTMLPartitionScanner.RHTML_RUBY_EXPRESSION);
		assistant.setContentAssistProcessor(rhtmlCp,
				RHTMLPartitionScanner.RHTML_TAG);

		assistant.enableAutoInsert(RailsUIPlugin.getInstance().getPreferenceStore().getBoolean(IRailsPreferenceConstants.RHTML_AUTOINSERT));

		assistant
				.setProposalPopupOrientation(IContentAssistant.CONTEXT_INFO_BELOW);
		assistant
				.setContextInformationPopupOrientation(IContentAssistant.CONTEXT_INFO_ABOVE);
		assistant.setContextInformationPopupBackground(RailsUIColorProvider
				.getInstance().getColor(new RGB(150, 150, 0)));

		return assistant;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.text.source.SourceViewerConfiguration#getTabWidth(org.eclipse.jface.text.source.ISourceViewer)
	 */
	public int getTabWidth(ISourceViewer sourceViewer) {
		return RailsUIPlugin.getInstance().getPreferenceStore().getInt(
				IRailsPreferenceConstants.RHTML_INDENTATION);
	}

}
