/*******************************************************************************
 * Copyright (c) 2005 RadRails.org and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.radrails.org/legal/cpl-v10.html
 *******************************************************************************/

package org.radrails.rails.internal.ui.editors.rhtml;

import org.eclipse.jface.text.DocumentCommand;
import org.eclipse.jface.text.IDocument;
import org.eclipse.jface.text.source.IOverviewRuler;
import org.eclipse.jface.text.source.IVerticalRuler;
import org.eclipse.jface.text.source.SourceViewerConfiguration;
import org.eclipse.jface.text.source.projection.ProjectionViewer;
import org.eclipse.jface.util.IPropertyChangeListener;
import org.eclipse.jface.util.PropertyChangeEvent;
import org.eclipse.swt.widgets.Composite;
import org.radrails.rails.internal.ui.RailsUIPlugin;
import org.radrails.rails.internal.ui.editors.TabExpander;
import org.radrails.rails.internal.ui.preferences.IRailsPreferenceConstants;

public class RHTMLSourceViewer extends ProjectionViewer {

	private boolean isTabReplacing = false;

	private boolean fIgnoreTextConverters = false;

	private TabExpander tabExpander;

	private IPropertyChangeListener propListener = new IPropertyChangeListener() {
		public void propertyChange(PropertyChangeEvent event) {
			if (event.getProperty().equals(
					IRailsPreferenceConstants.RHTML_FORMAT_USE_TAB)) {
				isTabReplacing = !RailsUIPlugin.getInstance()
						.getPreferenceStore().getBoolean(
								IRailsPreferenceConstants.RHTML_FORMAT_USE_TAB);
				if (isTabReplacing) {
					int length = RailsUIPlugin.getInstance()
							.getPreferenceStore().getInt(
									IRailsPreferenceConstants.RHTML_INDENTATION);
					updateIndentPrefixes(length);
				} else {
					updateIndentPrefixes();
				}
			} else if (event.getProperty().equals(
					IRailsPreferenceConstants.RHTML_INDENTATION)) {
				int length = RailsUIPlugin.getInstance().getPreferenceStore()
						.getInt(IRailsPreferenceConstants.RHTML_INDENTATION);
				tabExpander = new TabExpander(length);
				updateIndentPrefixes(length);
			}
		}
	};

	public RHTMLSourceViewer(Composite composite, IVerticalRuler verticalRuler,
			IOverviewRuler overviewRuler, boolean overviewRulerVisible,
			int styles) {
		super(composite, verticalRuler, overviewRuler, overviewRulerVisible,
				styles);
		initializeTabReplace();
		RailsUIPlugin.getInstance().getPreferenceStore()
				.addPropertyChangeListener(propListener);
	}

	public void configure(SourceViewerConfiguration configuration) {
		super.configure(configuration);
		if (isTabReplacing) {
			int length = RailsUIPlugin.getInstance().getPreferenceStore()
					.getInt(IRailsPreferenceConstants.RHTML_INDENTATION);
			updateIndentPrefixes(length);
		} else {
			updateIndentPrefixes();
		}
	}

	public void doOperation(int operation) {
		if (getTextWidget() == null || !redraws()) {
			return;
		}

		switch (operation) {
		case UNDO:
			fIgnoreTextConverters = true;
			break;
		case REDO:
			fIgnoreTextConverters = true;
			break;
		}

		super.doOperation(operation);
	}

	protected void customizeDocumentCommand(DocumentCommand command) {
		super.customizeDocumentCommand(command);
		if (!fIgnoreTextConverters) {
			convertTabs(command, getDocument());
		}
		fIgnoreTextConverters = false;
	}

	void initializeTabReplace() {
		this.isTabReplacing = !RailsUIPlugin.getInstance().getPreferenceStore()
				.getBoolean(IRailsPreferenceConstants.RHTML_FORMAT_USE_TAB);
		if (this.isTabReplacing) {
			int length = RailsUIPlugin.getInstance().getPreferenceStore()
					.getInt(IRailsPreferenceConstants.RHTML_INDENTATION);
			tabExpander = new TabExpander(length);
		}
	}

	protected void convertTabs(DocumentCommand command, IDocument document) {
		if (!isTabReplacing)
			return;

		if (command.text.equals("\t"))
			tabExpander.expandTab(command, document);
	}

	public boolean isTabReplacing() {
		return isTabReplacing;
	}

	public String getIndentString() {
		return tabExpander.getFullIndent();
	}

	private void updateIndentPrefixes() {
		String[] prefixes = new String[] { "\t", " ", "" };
		setIndentPrefixesForContentTypes(prefixes);
	}

	private void updateIndentPrefixes(int spacesPerTab) {
		StringBuffer spaces = new StringBuffer();
		for (int i = 0; i < spacesPerTab; i++) {
			spaces.append(" ");
		}
		String[] prefixes = new String[] { spaces.toString(), "\t", "" };
		setIndentPrefixesForContentTypes(prefixes);
	}

	private void setIndentPrefixesForContentTypes(String[] prefixes) {
		setIndentPrefixes(prefixes, IDocument.DEFAULT_CONTENT_TYPE);
		String[] rhtmlTypes = RHTMLPartitionScanner.RHTML_PARTITION_TYPES;
		for (int i = 0; i < rhtmlTypes.length; i++) {
			setIndentPrefixes(prefixes, rhtmlTypes[i]);
		}
	}

}
