/*******************************************************************************
 * Copyright (c) 2005 RadRails.org and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.radrails.org/legal/cpl-v10.html
 *******************************************************************************/
package org.radrails.rails.internal.ui.editors.rhtml;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.jface.text.IDocument;
import org.eclipse.jface.text.TextAttribute;
import org.eclipse.jface.text.rules.IToken;
import org.eclipse.jface.text.rules.ITokenScanner;
import org.eclipse.jface.text.rules.Token;
import org.eclipse.jface.util.PropertyChangeEvent;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.RGB;
import org.eclipse.ui.editors.text.EditorsUI;
import org.eclipse.ui.texteditor.ChainedPreferenceStore;
import org.radrails.rails.internal.ui.RailsUIColorProvider;
import org.radrails.rails.internal.ui.editors.CompositeTokenScanner;
import org.rubypeople.rdt.core.RubyCore;
import org.rubypeople.rdt.internal.ui.RubyPlugin;
import org.rubypeople.rdt.internal.ui.text.IRubyColorConstants;
import org.rubypeople.rdt.internal.ui.text.PreferencesAdapter;
import org.rubypeople.rdt.internal.ui.text.RubyCommentScanner;
import org.rubypeople.rdt.internal.ui.text.RubyPartitionScanner;
import org.rubypeople.rdt.internal.ui.text.ruby.AbstractRubyScanner;
import org.rubypeople.rdt.internal.ui.text.ruby.RubyCodeScanner;
import org.rubypeople.rdt.internal.ui.text.ruby.SingleTokenRubyCodeScanner;
import org.rubypeople.rdt.ui.text.IColorManager;

/**
 * Wrapper token scanner for Ruby expressions in the RHTML file. Delegates calls
 * to a scanner from RDT. Replaces the background color of all tokens to the one
 * set in the preferences.
 * 
 * @author mkent
 * 
 */
public class RubyExpressionScanner extends CompositeTokenScanner {

	private Color fBgColor;

	/**
	 * Constructor.
	 */
	public RubyExpressionScanner() {
		super(new RubyPartitionScanner());

		// Load background color for expressions
		fBgColor = RailsUIColorProvider.getInstance().getColorFromPreference(
				IRHTMLEditorColorConstants.RUBY_EXPRESSION_BACKGROUND);
	}

	protected Map createTokenScanners() {
		Map tokenScanners = super.createTokenScanners();
		
		// Create chained pref store for RDT token scanners
		List stores = new ArrayList();
		stores.add(RubyPlugin.getDefault().getPreferenceStore());
		stores.add(new PreferencesAdapter(RubyCore.getPlugin()
				.getPluginPreferences()));
		stores.add(EditorsUI.getPreferenceStore());

		IPreferenceStore prefStore = new ChainedPreferenceStore(
				(IPreferenceStore[]) stores.toArray(new IPreferenceStore[stores
						.size()]));

		IColorManager colorManager = RubyPlugin.getDefault().getRubyTextTools()
				.getColorManager();

		// Create RDT token scanners
		ITokenScanner codeScanner = new RubyCodeScanner(colorManager, prefStore);
		ITokenScanner multilineCommentScanner = new RubyCommentScanner(
				colorManager, prefStore,
				IRubyColorConstants.RUBY_MULTI_LINE_COMMENT);
		ITokenScanner singlelineCommentScanner = new RubyCommentScanner(
				colorManager, prefStore,
				IRubyColorConstants.RUBY_SINGLE_LINE_COMMENT);
		ITokenScanner stringScanner = new SingleTokenRubyCodeScanner(
				colorManager, prefStore, IRubyColorConstants.RUBY_STRING);
		ITokenScanner regexpScanner = new SingleTokenRubyCodeScanner(
				colorManager, prefStore, IRubyColorConstants.RUBY_REGEXP);
		ITokenScanner commandScanner = new SingleTokenRubyCodeScanner(
				colorManager, prefStore, IRubyColorConstants.RUBY_COMMAND);

		// Map token scanners to content types
		tokenScanners.put(IDocument.DEFAULT_CONTENT_TYPE, codeScanner);
		tokenScanners.put(RubyPartitionScanner.RUBY_MULTI_LINE_COMMENT,
				multilineCommentScanner);
		tokenScanners.put(RubyPartitionScanner.RUBY_SINGLE_LINE_COMMENT,
				singlelineCommentScanner);
		tokenScanners.put(RubyPartitionScanner.RUBY_STRING, stringScanner);
		tokenScanners.put(RubyPartitionScanner.RUBY_REGULAR_EXPRESSION,
				regexpScanner);
		tokenScanners.put(RubyPartitionScanner.RUBY_COMMAND, commandScanner);
		return tokenScanners;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.text.rules.ITokenScanner#nextToken()
	 */
	public IToken nextToken() {
		IToken ctok = super.nextToken();
		
		// Apply background color
		Object tdata = ctok.getData();
		if (tdata instanceof TextAttribute) {
			TextAttribute ta = (TextAttribute) ctok.getData();
			ctok = new Token(new TextAttribute(ta.getForeground(), fBgColor, ta
					.getStyle()));
		}

		return ctok;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.radrails.rails.internal.ui.editors.IEditorPropertyChangeListener#applyPropertyChange(org.eclipse.jface.util.PropertyChangeEvent)
	 */
	public void applyPropertyChange(PropertyChangeEvent event) {
		
	}
	
	/**
	 * Determines whether the given event affects the text presentation of the
	 * token scanners.
	 * 
	 * @param event
	 *            the property change event
	 * @return true if the presentation is affected, false otherwise
	 */
	public boolean affectsBehavior(PropertyChangeEvent event) {
		boolean affected = false;
		Iterator i = fTokenScanners.values().iterator();
		while (i.hasNext() && !affected) {
			AbstractRubyScanner ars = (AbstractRubyScanner) i.next();
			affected = affected || ars.affectsBehavior(event);
		}
		return affected || internalAffectsBehavior(event);
	}
	
	private boolean internalAffectsBehavior(PropertyChangeEvent event) {
		return event.getProperty().equals(IRHTMLEditorColorConstants.RUBY_EXPRESSION_BACKGROUND);
	}

	/**
	 * Reacts to a property change event by applying the event to each token
	 * scanner.
	 * 
	 * @param event
	 *            the property change event
	 */
	public void adaptToPreferenceChange(PropertyChangeEvent event) {
		Iterator i = fTokenScanners.values().iterator();
		while (i.hasNext()) {
			AbstractRubyScanner ars = (AbstractRubyScanner) i.next();
			if (ars.affectsBehavior(event)) {
				ars.adaptToPreferenceChange(event);
			}
		}
		
		if (internalAffectsBehavior(event)) {
			internalAdaptToPreferenceChange(event);
		}
	}
	
	private void internalAdaptToPreferenceChange(PropertyChangeEvent event) {
		if(	event.getNewValue() instanceof String) {
			fBgColor = RailsUIColorProvider.getInstance().getColor(
					(String) event.getNewValue());
		} else {
			fBgColor = RailsUIColorProvider.getInstance().getColor(
					(RGB) event.getNewValue());
		}
	}
}
