/*******************************************************************************
 * Copyright (c) 2005 RadRails.org and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.radrails.org/legal/cpl-v10.html
 *******************************************************************************/

package org.radrails.rails.internal.ui.preferences;

import java.io.File;
import java.util.ArrayList;
import java.util.List;

import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Path;
import org.eclipse.core.runtime.Status;
import org.eclipse.jface.dialogs.StatusDialog;
import org.eclipse.jface.preference.PreferencePage;
import org.eclipse.jface.viewers.ArrayContentProvider;
import org.eclipse.jface.viewers.CheckStateChangedEvent;
import org.eclipse.jface.viewers.CheckboxTableViewer;
import org.eclipse.jface.viewers.ICheckStateListener;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.layout.RowData;
import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.DirectoryDialog;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.swt.widgets.TableItem;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchPreferencePage;
import org.radrails.rails.core.gems.Gem;
import org.radrails.rails.core.gems.GemsManager;
import org.radrails.rails.internal.ui.RailsUIPlugin;

/**
 * Preference page for managing Rails installations on the system.
 * 
 * @author mkent
 * 
 */
public abstract class GemsManagerPreferencePage extends PreferencePage implements
		IWorkbenchPreferencePage {

	protected CheckboxTableViewer tableViewer;

	protected Button addButton;

	protected Button editButton;

	protected Button removeButton;

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.IWorkbenchPreferencePage#init(org.eclipse.ui.IWorkbench)
	 */
	public void init(IWorkbench workbench) {
		// Do nothing
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.preference.PreferencePage#createContents(org.eclipse.swt.widgets.Composite)
	 */
	protected Control createContents(Composite parent) {
		noDefaultAndApplyButton();

		// Create the main page layout
		Composite page = new Composite(parent, SWT.NULL);
		GridLayout layout = new GridLayout();
		layout.numColumns = 2;
		page.setLayout(layout);

		// Create the table and buttons
		Table table = createGemsTable(page);
		createGemsTableViewer(table);
		createButtonGroup(page);

		// Populate the table with the current rails installs
		tableViewer.setInput(getGemsManager().getGems());
		Gem selectedGem = getGemsManager().getSelectedGem();
		if (selectedGem != null)
			tableViewer.setChecked(selectedGem, true);

		enableButtons();

		return page;
	}
	
	protected abstract GemsManager getGemsManager();

	/**
	 * Creates the table for the gems.
	 * 
	 * @param composite
	 *            the parent composite
	 * @return a new Table widget
	 */
	protected Table createGemsTable(Composite composite) {
		Table table = new Table(composite, SWT.CHECK | SWT.BORDER
				| SWT.FULL_SELECTION);

		// Set the layout info
		GridData data = new GridData(GridData.FILL_BOTH);
		table.setLayoutData(data);
		table.setHeaderVisible(true);
		table.setLinesVisible(false);

		// Create the columns
		TableColumn column = new TableColumn(table, SWT.NULL);
		column.setText("Name");
		column.setWidth(125);

		column = new TableColumn(table, SWT.NULL);
		column.setText("Location");
		column.setWidth(350);

		return table;
	}

	/**
	 * Create a viewer for the gems table.
	 * 
	 * @param table
	 *            the table to view
	 */
	protected void createGemsTableViewer(Table table) {
		tableViewer = new CheckboxTableViewer(table);

		tableViewer.setLabelProvider(new GemsLabelProvider());
		tableViewer.setContentProvider(new ArrayContentProvider());

		// Add action listeners
		tableViewer
				.addSelectionChangedListener(new ISelectionChangedListener() {
					public void selectionChanged(SelectionChangedEvent evt) {
						enableButtons();
					}
				});

		tableViewer.addCheckStateListener(new ICheckStateListener() {
			public void checkStateChanged(CheckStateChangedEvent event) {
				updateSelectedGem(event.getElement());
			}
		});
	}

	/**
	 * Create the buttons for the page.
	 * 
	 * @param composite
	 *            the parent composite
	 */
	protected void createButtonGroup(Composite composite) {
		Composite buttons = new Composite(composite, SWT.NULL);

		// Set the layout data for the button group
		buttons.setLayoutData(new GridData(GridData.VERTICAL_ALIGN_BEGINNING));
		GridLayout layout = new GridLayout();
		layout.marginHeight = 0;
		layout.marginWidth = 0;
		buttons.setLayout(layout);

		// Create the Add button
		addButton = new Button(buttons, SWT.PUSH);
		addButton.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		addButton.setText("Add");
		addButton.addListener(SWT.Selection, new Listener() {
			public void handleEvent(Event evt) {
				try {
					addGem();
				} catch (Exception e) {
					e.printStackTrace();
				}
			}
		});

		// Create the Edit button
		editButton = new Button(buttons, SWT.PUSH);
		editButton.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		editButton.setText("Edit");
		editButton.addListener(SWT.Selection, new Listener() {
			public void handleEvent(Event evt) {
				editGem();
			}
		});

		// Create the Remove button
		removeButton = new Button(buttons, SWT.PUSH);
		removeButton.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		removeButton.setText("Remove");
		removeButton.addListener(SWT.Selection, new Listener() {
			public void handleEvent(Event evt) {
				removeGem();
			}
		});
	}

	/**
	 * Adds a new gem to the table. Shows an edit dialog to solicit
	 * information from the user.
	 */
	protected void addGem() {
		Gem g = new Gem();

		// Show an edit dialog with an empty install object
		EditGemDialog dialog = new EditGemDialog(getShell(),
				"Edit Gem", g);
		dialog.create();

		// Add gem to table and select it if nothing else is checked
		if (EditGemDialog.OK == dialog.open()) {
			tableViewer.add(g);
			Object[] checked = tableViewer.getCheckedElements();
			if (checked == null || checked.length == 0) {
				tableViewer.setChecked(g, true);
			}
		}
	}

	/**
	 * Edits a gem in the table. Shows an edit dialog to change the
	 * properties of the gem.
	 */
	protected void editGem() {
		Gem g = getGemsManager().getSelectedGem();
		EditGemDialog editor = new EditGemDialog(getShell(),
				"Edit Gem", g);
		editor.create();

		if (EditGemDialog.OK == editor.open())
			tableViewer.update(g, null);
	}

	/**
	 * Removes a gem from the table.
	 */
	protected void removeGem() {
		tableViewer.remove(getSelectedGem());
	}

	/**
	 * Enables the page buttons based on the table selection.
	 */
	protected void enableButtons() {
		if (getSelectedGem() != null) {
			editButton.setEnabled(true);
			removeButton.setEnabled(true);
		} else {
			editButton.setEnabled(false);
			removeButton.setEnabled(false);
		}
	}

	/**
	 * Updates the selected gem in the table.
	 * 
	 * @param gem
	 *            the gem to update
	 */
	protected void updateSelectedGem(Object gem) {
		Object[] checkedElements = tableViewer.getCheckedElements();
		for (int i = 0; i < checkedElements.length; i++) {
			tableViewer.setChecked(checkedElements[i], false);
		}

		tableViewer.setChecked(gem, true);
	}

	/**
	 * Returns the selected gem from the table.
	 * 
	 * @return the selected gem
	 */
	protected Gem getSelectedGem() {
		IStructuredSelection selection = (IStructuredSelection) tableViewer
				.getSelection();
		return (Gem) selection.getFirstElement();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.preference.IPreferencePage#performOk()
	 */
	public boolean performOk() {
		TableItem[] tableItems = tableViewer.getTable().getItems();
		List gems = new ArrayList(tableItems.length);
		for (int i = 0; i < tableItems.length; i++) {
			gems.add(tableItems[i].getData());
		}
		getGemsManager().setGems(gems);

		Object[] checkedElements = tableViewer.getCheckedElements();
		if (checkedElements.length > 0) {
			getGemsManager().setSelectedGem((Gem) checkedElements[0]);
		}
		else {
			getGemsManager().setSelectedGem(null);
		}

		return super.performOk();
	}

	/**
	 * A simple edit dialog used to edit properties of a Rails install. Contains
	 * fields to edit the name and location of the install. Also has a status
	 * bar to display validation results.
	 * 
	 * @author mkent
	 * 
	 */
	private class EditGemDialog extends StatusDialog {
		private Gem theGem;

		private Text gemNameText;

		private Text gemLocationText;

		public EditGemDialog(Shell parentShell, String title,
				Gem gem) {
			super(parentShell);
			setTitle(title);
			theGem = gem;
		}

		/**
		 * Creates the location text field.
		 * 
		 * @param composite
		 *            the parent composite
		 */
		protected void createLocationField(Composite composite) {
			new Label(composite, SWT.NONE).setText("Path");

			// Set the layout data
			Composite locationComposite = new Composite(composite, SWT.NONE);
			RowLayout locationLayout = new RowLayout();
			locationLayout.marginLeft = 0;
			locationComposite.setLayout(locationLayout);

			// Create the text field
			gemLocationText = new Text(locationComposite, SWT.SINGLE
					| SWT.BORDER);
			gemLocationText.addModifyListener(new ModifyListener() {
				public void modifyText(ModifyEvent e) {

					File path = new File(gemLocationText.getText());
					if (path.isDirectory()) {
						updateStatus(new Status(IStatus.OK, RailsUIPlugin
								.getInstance().getBundle().getSymbolicName(),
								0, "", null));
					} else {
						updateStatus(new Status(IStatus.ERROR, RailsUIPlugin
								.getInstance().getBundle().getSymbolicName(),
								0, "Invalid path", null));
					}
				}
			});
			gemLocationText.setLayoutData(new RowData(120, SWT.DEFAULT));

			// Create the browse button
			Button browseButton = new Button(composite, SWT.PUSH);
			browseButton.setLayoutData(new GridData(
					GridData.HORIZONTAL_ALIGN_END));
			browseButton.setText("Browse");
			browseButton.addSelectionListener(new SelectionAdapter() {
				public void widgetSelected(SelectionEvent e) {
					DirectoryDialog dialog = new DirectoryDialog(getShell());
					dialog.setFilterPath(gemLocationText.getText());
					dialog
							.setMessage("Select the gem directory:");
					dialog.setText("Location Selection");
					String newPath = dialog.open();
					if (newPath != null)
						gemLocationText.setText(newPath);
				}
			});
		}

		/**
		 * Creates the name text field.
		 * 
		 * @param composite
		 *            the parent composite
		 */
		protected void createNameField(Composite composite) {
			new Label(composite, SWT.NONE).setText("Name");

			// Set the layout data
			GridData gridData = new GridData(GridData.HORIZONTAL_ALIGN_FILL);
			gridData.horizontalSpan = 2;

			// Create the text field
			gemNameText = new Text(composite, SWT.SINGLE | SWT.BORDER);
			gemNameText.addModifyListener(new ModifyListener() {
				public void modifyText(ModifyEvent e) {
					if (gemNameText.getText().equals("")) {
						updateStatus(new Status(IStatus.ERROR, RailsUIPlugin
								.getInstance().getBundle().getSymbolicName(),
								0, "Name cannot be empty", null));
					} else {
						updateStatus(new Status(IStatus.OK, RailsUIPlugin
								.getInstance().getBundle().getSymbolicName(),
								0, "", null));
					}
				}
			});
			gemNameText.setLayoutData(gridData);
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see org.eclipse.jface.dialogs.Dialog#okPressed()
		 */
		protected void okPressed() {
			if (theGem == null) {
				theGem = new Gem();
			}

			theGem.setName(gemNameText.getText());
			theGem.setLocation(new Path(gemLocationText.getText()));
			super.okPressed();
		}

		/*
		 * (non-Javadoc)
		 * 
		 * @see org.eclipse.jface.dialogs.Dialog#createDialogArea(org.eclipse.swt.widgets.Composite)
		 */
		protected Control createDialogArea(Composite parent) {
			Composite composite = (Composite) super.createDialogArea(parent);

			// Set the layout data
			GridLayout layout = new GridLayout();
			layout.numColumns = 3;
			composite.setLayout(layout);

			// Create the text fields
			createNameField(composite);
			createLocationField(composite);

			// Pre-load the fields with the selected install
			String installName = theGem.getName();
			gemNameText.setText(installName != null ? installName : "");

			IPath installLocation = theGem.getLocation();
			gemLocationText
					.setText(installLocation != null ? installLocation
							.toOSString() : "");

			return composite;
		}
	}

}
