/*******************************************************************************
 * Copyright (c) 2005 RadRails.org and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.radrails.org/legal/cpl-v10.html
 *******************************************************************************/

package org.radrails.rails.internal.ui.tail;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Device;
import org.eclipse.swt.widgets.Display;
import org.eclipse.ui.console.MessageConsole;
import org.eclipse.ui.console.MessageConsoleStream;
import org.radrails.rails.internal.ui.RailsUILog;
import org.radrails.rails.internal.ui.tail.ANSI.Parser;
import org.radrails.rails.internal.ui.tail.ANSI.Token;

/**
 * Acts similar to the UNIX 'tail' command which allows watching a file
 * update in real time.
 *
 * @author	mbaumbach
 *
 * @version	0.5.0
 */
public class Tail {

    /**
     * The file to tail.
     */
    private File file;
    
    /**
     * Keeps track of the last time the file was modified.
     */
    private long fileLength;
    
    /**
     * The BufferedReader used to read the file.
     */
    private BufferedReader in;
    
    /**
     * Used for outputing the tail output to.
     */
    private MessageConsoleStream out;
    
    /**
     * The MessageConsole this tail is part of.
     */
    private MessageConsole console;
    
    /**
     * The current Device.
     */
    private Device dev = Display.getCurrent();
    
    /**
     * Specifies if the thread should continue running or not.
     */
    private boolean continueRunning;
    
    /**
     * The number of characters to display on opening a file.
     */
    private static final int NUM_CHARS_START = 400;
    
    /**
     * Creates a new Tail object for the specified filename.
     * 
     * @param   filename    The File to tail.
     */
    public Tail(File filename) {
        this.file = filename;
        fileLength = 0;
        continueRunning = false;
    }
    
    /**
     * Prints out the file's contents in real time.
     */
    public void start(MessageConsole console) {
        // Setup the reader for the file and start the file length thread
        try {
            in = new BufferedReader(new FileReader(file));
        } catch (FileNotFoundException e) {
        	RailsUILog.logError("Error opening reader on tailed file", e);
        }
        this.console = console;
        continueRunning = true;
        
        // If the file is long, skip to the last few lines
        if(file.length() > NUM_CHARS_START) {
        	try {
				in.skip(file.length() - NUM_CHARS_START);
			} catch (IOException e) {
				RailsUILog.logError("Error skipping to end of tailed file", e);
			}
        }
        startFileLengthThread();
    }
    
    /**
     * Reads the latest information from a file and outputs it to the screen.
     */
    private void read() {
        // As long as we have a reader for the file, read in the latest
        if ( in != null ) {
            List tokens;
            // looking up the right stream should be faster than creating a new one
            // for each token
            Map outputStreams = new HashMap();
            Token token;
            try {
                while( (tokens = Parser.parse(in.readLine())) != null ) {
                    Iterator it = tokens.iterator();
                	while (it.hasNext()) {
                		token = (Token)it.next();
                		if (outputStreams.containsKey(token.getAnsi())) {
                			out = (MessageConsoleStream)outputStreams.get(token.getAnsi());
                		} else {
                			out = new MessageConsoleStream(console);
                			if (token.hasForegroundColor())
                				out.setColor(new Color(dev, token.getForegroundRGB()));
                			if (token.hasFontStyle())
                				out.setFontStyle(token.getFontStyle());
                			outputStreams.put(token.getAnsi(), out);
                		}
                	
                			
                		out.print(token.toString());	
                	}
                	out.println();
                }
            } catch (IOException e) {
            	RailsUILog.logError("Error reading from tailed file", e);
            }
        }
    }
    
    /**
     * Starts a thread that checks the length of a file. When the file size changes,
     * it calls read() so that the new changes are printed.
     */
    private void startFileLengthThread() {
        Thread t = new Thread(new Runnable() {
           public void run() {
               // Run forever
               while( continueRunning ) {
                   // If the file length has changed, update the file length and print the new info
                   if ( file.length() > fileLength ) {
                       fileLength = file.length();
                       read();
                   }
                   // Sleep for a moment so that we can read everything in
                   try {
                       Thread.sleep(10);
                   } catch (InterruptedException e) {
                	   RailsUILog.logError("Error sleeping tail thread", e);
                   }
               }
           }
        });
        t.start();
    }
    
    /**
     * Stops the tail program.
     */
    public void stop() {
        continueRunning = false;
    }
    
    public boolean isStopped() {
    	return !continueRunning;
    }
    
} // Tail