/*******************************************************************************
 * Copyright (c) 2005 RadRails.org and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.radrails.org/legal/cpl-v10.html
 *******************************************************************************/

package org.radrails.rails.internal.ui.wizards;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;
import java.lang.reflect.InvocationTargetException;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IProjectDescription;
import org.eclipse.core.resources.IWorkspace;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.Platform;
import org.eclipse.core.runtime.SubProgressMonitor;
import org.eclipse.debug.core.ILaunchConfiguration;
import org.eclipse.debug.core.ILaunchConfigurationWorkingCopy;
import org.eclipse.debug.core.ILaunchManager;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.operation.IRunnableWithProgress;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.wizard.Wizard;
import org.eclipse.ui.INewWizard;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.actions.WorkspaceModifyDelegatingOperation;
import org.eclipse.ui.plugin.AbstractUIPlugin;
import org.eclipse.ui.wizards.newresource.BasicNewResourceWizard;
import org.radrails.internal.core.launching.IRailsLaunchConfigAttributes;
import org.radrails.rails.core.RailsLog;
import org.radrails.rails.internal.core.RailsRuntime;
import org.radrails.rails.internal.ui.IRailsUIConstants;
import org.radrails.rails.internal.ui.RailsUILog;
import org.radrails.rails.internal.ui.RailsUIPlugin;
import org.radrails.rails.internal.ui.wizards.pages.WizardNewRailsProjectPage;
import org.radrails.server.core.Server;
import org.radrails.server.core.ServerManager;
import org.radrails.server.core.mongrel.MongrelServer;
import org.radrails.server.core.webrick.WEBrickServer;
import org.rubypeople.rdt.core.RubyCore;

/**
 * This wizard creates a new Rails project.
 * 
 * @author mkent
 * 
 */
public class NewRailsProjectWizard extends Wizard implements INewWizard {

	private static final String MSG_ERROR_DISABLING_PLURALIZATION = "Error disabling table pluralization";

	private IProject newProject;

	private WizardNewRailsProjectPage page1;

	private IWorkbench workbench;
	
	/**
	 * Default constructor.
	 */
	public NewRailsProjectWizard() {
		setNeedsProgressMonitor(true);
		setWindowTitle("New Rails project");
	}

	/**
	 * @see org.eclipse.jface.wizard.IWizard#performFinish()
	 */
	public boolean performFinish() {
		boolean success = true;

		IRunnableWithProgress newProjectOp = new WorkspaceModifyDelegatingOperation(
				createNewProjectOpRunnable());

		// Run the project creation operation
		try {
			getContainer().run(false, true, newProjectOp);
		} catch (InvocationTargetException e) {
			RailsUILog.logError("Error creating project", e);
			success = false;
		} catch (InterruptedException e) {
			RailsUILog.logError("Error creating project", e);
			success = false;
		}

		// Select and reveal the project in the workbench window
		BasicNewResourceWizard.selectAndReveal(newProject, workbench
				.getActiveWorkbenchWindow());
		return success;
	}

	/**
	 * Create a runnable new project operation.
	 * 
	 * @return a runnable new project operation
	 */
	private IRunnableWithProgress createNewProjectOpRunnable() {
		return new IRunnableWithProgress() {
			public void run(IProgressMonitor monitor)
					throws InvocationTargetException, InterruptedException {
				monitor.beginTask("Creating project...", 40);

				// Create the project description
				IWorkspace workspace = ResourcesPlugin.getWorkspace();
				newProject = page1.getProjectHandle();
				IProjectDescription description = workspace
						.newProjectDescription(newProject.getName());

				// Set the location path of the project
				IPath oldPath = Platform.getLocation();
				IPath newPath = page1.getLocationPath();
				if (!oldPath.equals(newPath)) {
					oldPath = newPath;
					description.setLocation(newPath);
				}

				// Add the Rails nature
				String[] natureIds = { IRailsUIConstants.RAILS_PROJECT_NATURE };
				description.setNatureIds(natureIds);

				// Create and open the project
				try {
					if (!newProject.exists()) {
						newProject.create(description, new SubProgressMonitor(
								monitor, 10));
					}
					if (!newProject.isOpen()) {
						newProject.open(new SubProgressMonitor(monitor, 10));
					}

					// Add the Ruby nature
					RubyCore.addRubyNature(newProject, new SubProgressMonitor(
							monitor, 10));

					// Generate the Rails skeleton if requested
					if (page1.getGenerateButtonSelection()) {

						// Run the rails command to create the project files
						runRailsCommand(newProject.getLocation().append("../")
								.toFile(), newProject.getName(), newProject.getLocation().toFile().getName(),
								new SubProgressMonitor(monitor, 10));
						
						// Disable table pluralization if requested
						if (page1.getPluralizationButtonSelection()) {
							disablePluralization();
						}
					}
				} catch (CoreException e) {
					throw new InvocationTargetException(e);
				} finally {
					monitor.done();
				}

				// Create a WEBrick server if requested
				if (page1.getWebrickButtonSelection()) {
					Server s = new WEBrickServer(newProject.getLocation()
							.toFile(), page1.getProjectName() + "Server");
					ServerManager.getInstance().addServer(s);
				}
				
				if(page1.getMongrelButtonSelection()) {
					Server s = new MongrelServer(newProject.getLocation().toFile());
					ServerManager.getInstance().addServer(s);
				}
			}
		};
	}

	/**
	 * Runs the rails command to create a new project.
	 * 
	 * @param location
	 *            the full path to the project root
	 * @param args
	 *            the arguments to the rails command
	 * @param monitor
	 *            the progress monitor for the operation
	 */
	private void runRailsCommand(File location, String projName, String appName,
			IProgressMonitor monitor) {
		try {
			ILaunchConfigurationWorkingCopy wc = RailsRuntime
					.createInternalLaunch(IRailsLaunchConfigAttributes.SCRIPT_RAILS, "rails");
			wc.setAttribute(IRailsLaunchConfigAttributes.PROGRAM_ARGS, appName);
			wc.setAttribute(IRailsLaunchConfigAttributes.WORKING_DIR, location.toString());
			wc.setAttribute(IRailsLaunchConfigAttributes.SCRIPT_TYPE,
					IRailsLaunchConfigAttributes.SCRIPT_RAILS);
			wc.setAttribute(IRailsLaunchConfigAttributes.PROJECT_NAME, projName);
			wc.setAttribute(IRailsLaunchConfigAttributes.REQUIRES_REFRESH, "true");

			ILaunchConfiguration config = wc.doSave();
			config.launch(ILaunchManager.RUN_MODE, null);
		} catch (CoreException e) {
			RailsLog.logError("Error running rake task", e);
		}
	}

	/**
	 * Adds a line to the environment.rb configuration file to disable table
	 * pluralization.
	 */
	private void disablePluralization() {
		String pathToFile = newProject.getLocation().append(
				"config/environment.rb").toFile().toString();
		String disablePluralizationLine = "ActiveRecord::Base.pluralize_table_names = false";
		try {
			List lines = new ArrayList();
			BufferedReader reader = new BufferedReader(new FileReader(
					pathToFile));
			String inLine = null;
			while ((inLine = reader.readLine()) != null) {
				lines.add(inLine);
			}
			reader.close();
			PrintWriter writer = new PrintWriter(new FileWriter(pathToFile));
			Iterator i = lines.iterator();
			while (i.hasNext()) {
				String outLine = (String) i.next();
				writer.println(outLine);
			}
			writer.println(disablePluralizationLine);
			writer.close();
		} catch (FileNotFoundException fnfe) {
			MessageDialog.openError(workbench.getDisplay().getActiveShell(),
					MSG_ERROR_DISABLING_PLURALIZATION, "File " + pathToFile
							+ " not found");
		} catch (IOException ioe) {
			MessageDialog.openError(workbench.getDisplay().getActiveShell(),
					MSG_ERROR_DISABLING_PLURALIZATION, ioe
							.getLocalizedMessage());
		}
	}

	/**
	 * @see org.eclipse.jface.wizard.IWizard#addPages()
	 */
	public void addPages() {
		super.addPages();

		ImageDescriptor image = AbstractUIPlugin.imageDescriptorFromPlugin(
				RailsUIPlugin.getInstance().getBundle().getSymbolicName(),
				"icons/newproj_wiz.gif");

		page1 = new WizardNewRailsProjectPage("new.rails.project1");
		page1.setTitle("Rails project");
		page1.setDescription("Create a new Rails project");
		page1.setImageDescriptor(image);
		addPage(page1);
	}

	/**
	 * @see org.eclipse.ui.IWorkbenchWizard#init(org.eclipse.ui.IWorkbench,
	 *      org.eclipse.jface.viewers.IStructuredSelection)
	 */
	public void init(IWorkbench workbench, IStructuredSelection selection) {
		this.workbench = workbench;
	}

}
