/*******************************************************************************
 * Copyright (c) 2005 RadRails.org and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.radrails.org/legal/cpl-v10.html
 *******************************************************************************/

package org.radrails.server.core;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStreamReader;
import java.net.URL;
import java.util.Observable;

import org.eclipse.ui.PlatformUI;
import org.radrails.db.core.IDatabaseConstants;

/**
 * This class represents a web server that is capable of running Rails
 * applications. It is an observable object, and is observed by the ServersView
 * class.
 * 
 * Instead of traditional <code>setXXX</code> methods, <code>updateXXX</code>
 * methods are provided. These update methods will set the attribute and then
 * notify all observers of the change.
 * 
 * Subclasses must implement the {@link #start() start} and {@link #stop() stop}
 * methods. Subclasses should also create static factory methods to create
 * instances of themselves, such as <code>createMyServer</code>.
 * 
 * @author mkent
 * 
 */
public abstract class Server extends Observable {

	protected File project;

	protected String name;

	protected String type;

	protected String port;

	protected String environment;

	protected String status;

	/**
	 * Default constructor.
	 */
	protected Server() {

	}

	/**
	 * Constructor. Status is set to <code>IServerConstants.STOPPED</code> by
	 * default.
	 * 
	 * @param projectName
	 *            the location of the project the server is associated with
	 * @param name
	 *            the name of the server
	 * @param type
	 *            the type of the server (use {@link IServerConstants TYPE_XXX})
	 * @param portNumber
	 *            the number of the port the server will operate on
	 * @param environment
	 *            the runtime environment of the server (use
	 *            {@link IDatabaseConstants ENV_XXX})
	 */
	protected Server(File project, String name, String type, String portNumber,
			String environment) {
		this.project = project;
		this.name = name;
		this.type = type;
		this.port = portNumber;
		this.environment = environment;
		this.status = IServerConstants.STOPPED;
	}

	/**
	 * @return the location of the project the server is associated with
	 */
	public File getProject() {
		return project;
	}

	/**
	 * @return the name of the server
	 */
	public String getName() {
		return name;
	}

	/**
	 * @return the port number that the server is available on
	 */
	public String getPort() {
		return port;
	}

	/**
	 * @return the type of the server
	 */
	public String getType() {
		return type;
	}

	/**
	 * @return the runtime environment of the server
	 */
	public String getEnvironment() {
		return environment;
	}

	/**
	 * @return the status of the server
	 */
	public String getStatus() {
		return status;
	}

	/**
	 * Sets the port number of the server and notifies observers of the change.
	 * 
	 * @param port
	 *            the number of the port the server operates on
	 */
	public void updatePort(String port) {
		this.port = port;
		setChanged();
		notifyObservers(IServerConstants.UPDATE);
	}

	/**
	 * Sets the name of the server and notifies observers of the change.
	 * 
	 * @param name
	 *            the name of the server
	 */
	public void updateName(String name) {
		this.name = name;
		setChanged();
		notifyObservers(IServerConstants.UPDATE);
	}

	/**
	 * Sets the environment of the server and notifies observers of the change.
	 * 
	 * Use {@link IDatabaseConstants ENV_XXX}
	 * 
	 * @param environment
	 *            the environment of the server
	 */
	public void updateEnvironment(String environment) {
		this.environment = environment;
		setChanged();
		notifyObservers(IServerConstants.UPDATE);
	}

	/**
	 * Sets the status of the server and notifies observers of the change.
	 * 
	 * Use <code>IServerConstants.STARTED</code> or
	 * <code>IServerConstants.STOPPED</code>
	 * 
	 * @param status
	 *            the status of the server
	 */
	public void updateStatus(String status) {
		this.status = status;
		setChanged();
		notifyObservers(IServerConstants.UPDATE);
	}

	/**
	 * Updates the server status in a separate thread.
	 */
	protected void syncUpdateStatus(final String status) {
		if (!PlatformUI.getWorkbench().getDisplay().isDisposed()) {
			PlatformUI.getWorkbench().getDisplay().syncExec(new Runnable() {
				public void run() {
					updateStatus(status);
				}
			});
		}
	}

	/**
	 * Sets the changed flag. Clients should call this method before calling the
	 * <code>notifyObservers()</code> method explicitly.
	 */
	public void touch() {
		setChanged();
	}

	/**
	 * @return true if the server's status is
	 *         <code>IServerConstants.STARTED</code>, false otherwise.
	 */
	public boolean isStarted() {
		return status.equals(IServerConstants.STARTED);
	}

	/**
	 * @return true if the server's status is
	 *         <code>IServerConstants.STARTING</code>, false otherwise.
	 */
	public boolean isStarting() {
		return status.equals(IServerConstants.STARTING);
	}

	/**
	 * @return true if the server's status is
	 *         <code>IServerConstants.STOPPED</code>, false otherwise.
	 */
	public boolean isStopped() {
		return status.equals(IServerConstants.STOPPED);
	}

	/**
	 * @return true if the server's status is
	 *         <code>IServerConstants.STOPPING</code>, false otherwise.
	 */
	public boolean isStopping() {
		return status.equals(IServerConstants.STOPPING);
	}

	/**
	 * Starts the server. Subclass implementations must call
	 * {@link #setChanged() setChanged} and
	 * {@link #notifyObservers() notifyObservers} after the server is started.
	 * Pass {<code>IServerConstants.UPDATE</code> as the second argument to
	 * {@link #notifyObservers() notifyObservers}.
	 */
	public abstract void start();

	/**
	 * Stops the server. Subclass implementations must call
	 * {@link #setChanged() setChanged} and
	 * {@link #notifyObservers() notifyObservers} after the server is started.
	 * Pass <code>IServerConstants.UPDATE</code> as the second argument to
	 * {@link #notifyObservers() notifyObservers}.
	 */
	public abstract void stop();

	public abstract void restart();
	
	/**
	 * Writes a script from plugin jar to plugin metadata folder within the
	 * workspace.
	 * 
	 * @param rubyFile -
	 *            The file to place on the filesystem
	 * @return Absolute path to specified script file
	 */
	protected String getRubyScriptPath(String rubyFile) {
		String directoryFile = ServerPlugin.getInstance().getStateLocation()
				.toOSString()
				+ File.separator + rubyFile;
		File pluginDirFile = new File(directoryFile);

		if (!pluginDirFile.exists()) {
			try {
				pluginDirFile.createNewFile();
				URL u = ServerPlugin.getInstance().getBundle().getEntry(
						"/ruby/" + rubyFile);
				BufferedReader input = new BufferedReader(
						new InputStreamReader(u.openStream()));
				FileWriter output = new FileWriter(pluginDirFile);
				String line;
				while ((line = input.readLine()) != null) {
					output.write(line);
					output.write('\n');
				}
				output.flush();
				output.close();
				input.close();
			} catch (IOException e) {
				ServerLog.logError(
						"Error copying script file from jar to metadata", e);
			}
		}

		String path = "";
		try {
			path = pluginDirFile.getCanonicalPath();
		} catch (IOException e) {
			ServerLog.logError("Error getting script file path", e);
		}
		return path;
	}

}
