/*******************************************************************************
 * Copyright (c) 2005 RadRails.org and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.radrails.org/legal/cpl-v10.html
 *******************************************************************************/

package org.radrails.server.core;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;
import java.io.Reader;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Observer;

import javax.xml.parsers.FactoryConfigurationError;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.parsers.SAXParserFactory;

import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IResourceChangeEvent;
import org.eclipse.core.resources.IResourceChangeListener;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.IPath;
import org.eclipse.ui.PlatformUI;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;
import org.xml.sax.XMLReader;

/**
 * Manages a collection of all servers. Operations on servers should be called
 * through this class, because it notifies any listening views of changes to the
 * server.
 * 
 * The <code>ServerManager</code> manages all of the servers in the workspace.
 * It is also the model represented by <code>ServersView</code>.
 * 
 * When a new <code>Server</code> has been created, it should be added to
 * <code>ServerManager</code> using {@link #addServer(Server) addServer}
 * method. Similarly, to remove a <code>Server</code>, use the
 * {@link #removeServer(Server) removeServer} method.
 * 
 * The <code>ServerManager</code> also maintains a list of
 * <code>Observer</code> objects. These are conceptually the observers of the
 * <code>ServerManager</code>, but actually the observers of the
 * <code>Server</code>s maintained by the <code>ServerManager</code>. As
 * such, the {@link #addServerObserver(Observer) addServerObserver} and
 * {@link #deleteServerObserver(Observer) deleteServerObserver} methods should
 * be used instead of the <code>addObserver</code> and
 * <code>deleteObserver</code> methods on <code>Server</code>.
 * 
 * @author mkent
 * 
 */
public class ServerManager implements IResourceChangeListener {

	private static ServerManager instance;

	private Map serverMap;

	private List serverObservers;

	/**
	 * Default constructor.
	 */
	private ServerManager() {
		serverMap = new HashMap();
		serverObservers = new ArrayList();
		ResourcesPlugin.getWorkspace().addResourceChangeListener(this);
	}

	/**
	 * @return the singleton instance of <code>ServerManager</code>.
	 */
	public static ServerManager getInstance() {
		if (instance == null) {
			instance = new ServerManager();
		}
		return instance;
	}

	/**
	 * Convenience method to stop all servers in the <code>ServerManager</code>.
	 */
	public void stopAll() {
		Iterator i = serverMap.values().iterator();
		while (i.hasNext()) {
			Server s = (Server) i.next();
			if (s.isStarted()) {
				s.stop();

				// Force the status to STOPPED
				// Needed on workbench shutdown because the view gets disposed
				// before the status can be updated
				s.status = IServerConstants.STOPPED;
			}
		}
	}

	/**
	 * @return the next available server port
	 */
	public String getNextAvailablePort() {
		String port = IServerConstants.DEFAULT_WEBRICK_PORT;
		boolean found = false;

		while (!found) {
			if (!portInUse(port)) {
				found = true;
			} else {
				port = String.valueOf(Integer.parseInt(port) + 1);
			}
		}
		return port;
	}

	/**
	 * Determines if the given port is currently in use by a <code>Server</code>.
	 * 
	 * @param port
	 *            the port to check
	 * @return true if the port is in use, false otherwise
	 */
	public boolean portInUse(String port) {
		boolean taken = false;
		Iterator i = serverMap.values().iterator();
		while (i.hasNext() && !taken) {
			Server s = (Server) i.next();
			taken = port.equals(s.getPort());
		}
		return taken;
	}

	/**
	 * Determines if the given project has a <code>Server</code> of the given
	 * type.
	 * 
	 * @param projectName
	 *            the name of the <code>Server</code> to check
	 * @param type
	 *            the type of <code>Server</code> to check for
	 * @return true if the project has the <code>Server</code>, false
	 *         otherwise
	 */
	public boolean projectHasServer(String projectName, String type) {
		boolean has = false;
		Iterator i = serverMap.values().iterator();
		while (i.hasNext() && !has) {
			Server s = (Server) i.next();
			has = (projectName.equals(s.getProject().getName()) && type
					.equals(s.getType()));
		}
		return has;
	}

	/**
	 * Adds a <code>Server</code> to the <code>ServerManager</code>.
	 * 
	 * @param server
	 *            the <code>Server</code> to add
	 */
	public void addServer(Server server) {
		serverMap.put(server.getName(), server);

		// Need to add the current observers to the new server
		Iterator i = serverObservers.iterator();
		while (i.hasNext()) {
			Observer o = (Observer) i.next();
			server.addObserver(o);
		}

		// Notify the observers that the server has been added
		server.touch();
		server.notifyObservers(IServerConstants.ADD);
	}

	/**
	 * Removes a <code>Server</code> from the <code>ServerManager</code>.
	 * 
	 * @param server
	 *            the <code>Server</code> to remove
	 */
	public void removeServer(Server server) {
		serverMap.remove(server.getName());

		// Notify the observers that the server has been removed
		server.touch();
		server.notifyObservers(IServerConstants.REMOVE);
	}

	/**
	 * Clients should not call this method. Exists only for use by ServersView.
	 * 
	 * @return the servers held by the manager
	 */
	public Collection getServers() {
		return serverMap.values();
	}

	/**
	 * Adds an <code>Observer</code> to the list of current server observers.
	 * 
	 * @param ob
	 *            the observer to add
	 */
	public void addServerObserver(Observer ob) {
		serverObservers.add(ob);

		// Add observer of all servers
		Iterator i = serverMap.values().iterator();
		while (i.hasNext()) {
			Server s = (Server) i.next();
			s.addObserver(ob);
		}
	}

	/**
	 * Deletes an <code>Observer</code> from the list of current server
	 * observers.
	 * 
	 * @param ob
	 *            the observer to delete
	 */
	public void deleteServerObserver(Observer ob) {
		serverObservers.remove(ob);

		// Remove observer of all servers
		Iterator i = serverMap.values().iterator();
		while (i.hasNext()) {
			Server s = (Server) i.next();
			s.deleteObserver(ob);
		}
	}

	/**
	 * @see org.eclipse.core.resources.IResourceChangeListener#resourceChanged(org.eclipse.core.resources.IResourceChangeEvent)
	 */
	public void resourceChanged(IResourceChangeEvent event) {
		// When a project is deleted, delete its servers
		if (event.getType() == IResourceChangeEvent.PRE_DELETE) {
			IProject project = (IProject) event.getResource();

			// Find all of this project's servers and remove them
			Iterator i = serverMap.values().iterator();
			while (i.hasNext()) {
				final Server s = (Server) i.next();
				if (project.getLocation().toFile().getName().equals(
						s.getProject().getName())) {
					// This method will be called from a separate thread, so
					// use syncExec
					PlatformUI.getWorkbench().getDisplay().syncExec(
							new Runnable() {
								public void run() {
									// Stop the server, then remove
									s.stop();
									removeServer(s);
								}
							});
				}
			}
		}
		// When a project is closed, stop its servers
		else if (event.getType() == IResourceChangeEvent.PRE_CLOSE) {
			IProject project = (IProject) event.getResource();

			// Find all of this project's servers and stop them
			Iterator i = serverMap.values().iterator();
			while (i.hasNext()) {
				final Server s = (Server) i.next();
				if (project.getLocation().toFile().getName().equals(
						s.getProject().getName())) {
					// This method will be called from a separate thread, so
					// use syncExec
					PlatformUI.getWorkbench().getDisplay().syncExec(
							new Runnable() {
								public void run() {
									s.stop();
								}
							});
				}
			}
		}
	}

	/**
	 * Saves all current server configurations to file in XML format. The status
	 * of each server is not saved, because this method is only used on
	 * workbench shutdown and each server is assumed to be stopped.
	 */
	public void saveServers() {
		File f = getConfigFile();
		PrintWriter out = null;
		try {
			out = new PrintWriter(new FileWriter(f));
		} catch (FileNotFoundException e) {
			ServerLog.logError("Servers config file not found", e);
		} catch (IOException e) {
			ServerLog.logError("Error opening servers config", e);
		}

		writeXML(out);
		out.close();
	}

	/**
	 * Discards all existing server configurations, then loads saved server
	 * configurations from file.
	 */
	public void loadServers() {
		try {
			Reader fileReader = new FileReader(getConfigFile());
			XMLReader reader = SAXParserFactory.newInstance().newSAXParser()
					.getXMLReader();
			ServerManagerContentHandler handler = new ServerManagerContentHandler();
			reader.setContentHandler(handler);
			reader.parse(new InputSource(fileReader));

			// Discard current servers
			serverMap.clear();

			// Load each saved server into the map
			Collection servers = handler.getServers();
			Iterator i = servers.iterator();
			while (i.hasNext()) {
				Server s = (Server) i.next();
				serverMap.put(s.getName(), s);
			}
		} catch (FileNotFoundException e) {
			// This is okay, will get thrown if no config exists yet
		} catch (SAXException e) {
			ServerLog.logError("Error parsing servers config file", e);
		} catch (ParserConfigurationException e) {
			ServerLog.logError("Error configuring XML parser", e);
		} catch (FactoryConfigurationError e) {
			ServerLog.logError("Error configuring parser factory", e);
		} catch (IOException e) {
			ServerLog.logError("Error reading servers config file", e);
		}
	}

	/**
	 * Writes each server configuration to file in XML format.
	 * 
	 * @param out
	 *            the writer to use
	 */
	private void writeXML(PrintWriter out) {
		out.println("<?xml version=\"1.0\" encoding=\"UTF-8\"?>");
		out.println("<server-manager>");
		Iterator i = serverMap.values().iterator();
		while (i.hasNext()) {
			Server s = (Server) i.next();
			out.println("<server type=\"" + s.getType() + "\">");
			out.println("<project>" + s.getProject().toString() + "</project>");
			out.println("<name>" + s.getName() + "</name>");
			out.println("<port>" + s.getPort() + "</port>");
			out
					.println("<environment>" + s.getEnvironment()
							+ "</environment>");
			out.println("</server>");
		}
		out.println("</server-manager>");
		out.flush();
	}

	/**
	 * Returns the configuration file to use for the servers. The file is
	 * located in the plugin state directory and called <code>servers.xml</code>.
	 * 
	 * @return the config file
	 */
	private File getConfigFile() {
		IPath loc = ServerPlugin.getInstance().getStateLocation().append(
				"servers.xml");
		File f = new File(loc.toOSString());
		return f;
	}
}
