/*******************************************************************************
 * Copyright (c) 2005 RadRails.org and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.radrails.org/legal/cpl-v10.html
 *******************************************************************************/
package org.radrails.server.core.lighttpd;

import java.io.File;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.StringTokenizer;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.debug.core.DebugException;
import org.eclipse.debug.core.DebugPlugin;
import org.eclipse.debug.core.ILaunch;
import org.eclipse.debug.core.ILaunchManager;
import org.eclipse.debug.core.IStreamListener;
import org.eclipse.debug.core.Launch;
import org.eclipse.debug.core.model.IProcess;
import org.eclipse.debug.core.model.IStreamMonitor;
import org.eclipse.swt.SWT;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.console.ConsolePlugin;
import org.eclipse.ui.console.IConsole;
import org.eclipse.ui.console.MessageConsoleStream;
import org.radrails.db.core.IDatabaseConstants;
import org.radrails.rails.internal.core.RailsRuntime;
import org.radrails.server.core.IServerConstants;
import org.radrails.server.core.Server;
import org.radrails.server.core.ServerConsole;
import org.radrails.server.core.ServerLog;
import org.radrails.server.core.ServerManager;

public class LightTPDServer extends Server {

	private IProcess serverProcess;

	private MessageConsoleStream consoleStream;
	
	private int pid = -1;
	
	/**
	 * Constructor.
	 * 
	 * @param project
	 *            the location of the project the server is associated with
	 * @param name
	 *            the name of the server
	 * @param portNumber
	 *            the number of the port the server runs on
	 */
	public LightTPDServer(File project, String name, String port,
			String environment) {
		super(project, name, IServerConstants.TYPE_LIGHTTPD, port, environment);
	}

	/**
	 * Constructor. Environment defaults to development.
	 * 
	 * @param project
	 *            the location of the project the server is associated with
	 * @param name
	 *            the name of the server
	 * @param port
	 *            the number of the port the server runs on
	 */
	public LightTPDServer(File project, String name, String port) {
		this(project, name, port, IDatabaseConstants.ENV_DEVELOPMENT);
	}

	/**
	 * Constructor. Name defaults to projectNameServer, port defaults to next
	 * available, and environment defaults to development.
	 * 
	 * @param project
	 *            the location of the project the server is associated with
	 */
	public LightTPDServer(File project) {
		this(project, project.getName() + "Server", ServerManager.getInstance()
				.getNextAvailablePort(), IDatabaseConstants.ENV_DEVELOPMENT);
	}

	/**
	 * Empty constructor for use by the loadServers() method in ServerManager.
	 */
	public LightTPDServer() {
		this(null, null, null, null);
	}
	
	public void start() {
		syncUpdateStatus(IServerConstants.STARTING);

		// Prepare the server commands
		ArrayList preCommandList = new ArrayList();
		preCommandList.add("-e puts('RAILSPID='+Process.pid.to_s)");
		ArrayList commandList = new ArrayList();
		commandList.add("script/server");
		commandList.add("lighttpd");
		
		try {
			// Start the server process
			Process proc = RailsRuntime.rubyExec(
					(String[])preCommandList.toArray(new String[]{}),
					(String[])commandList.toArray(new String[]{}), project, false);
			ILaunch launch = new Launch(null, ILaunchManager.RUN_MODE, null);
			serverProcess = DebugPlugin.newProcess(launch, proc, "command");

			// Create a new console page for the server process
			SimpleDateFormat dateFormat = new SimpleDateFormat(
					"MMM d, yyyy h:mm:ss aa");
			String date = dateFormat.format(new Date());

			ServerConsole myConsole = new ServerConsole(this, project.getName()
					+ " - LightTPD Server - (" + date + ")", null);
			consoleStream = myConsole.newMessageStream();
			consoleStream.setColor(PlatformUI.getWorkbench().getDisplay()
					.getSystemColor(SWT.COLOR_BLUE));
			ConsolePlugin.getDefault().getConsoleManager().addConsoles(
					new IConsole[] { myConsole });

			// Prepare the stream listeners
			IStreamMonitor stdOut = serverProcess.getStreamsProxy()
					.getOutputStreamMonitor();
			IStreamMonitor stdErr = serverProcess.getStreamsProxy()
					.getErrorStreamMonitor();

			// Write stdout to the console
			stdOut.addListener(new IStreamListener() {
				public void streamAppended(final String text,
						IStreamMonitor monitor) {
					if (text.startsWith("RAILSPID")) {
						// Grab the process pid from the output
						StringTokenizer st = new StringTokenizer(text);
						while (st.hasMoreTokens()) {
							String token = st.nextToken();
							if (token.startsWith("RAILSPID=")) {
								String[] pidToken = token.split("=");
								pid = Integer.parseInt(pidToken[1]);
								System.out.println(pid);
							}
						}
					}else{
						if (text.indexOf("Rails application started") > -1) {  // *** Put the line that identifies the server is started (like WEBrick::HTTPServer#start
							// Notify the view
							syncUpdateStatus(IServerConstants.STARTED);
						}
						
						// Log the output to the console
						consoleStream.print(text);
					}
				}
			});

			// Write stderr to the console
			stdErr.addListener(new IStreamListener() {
				public void streamAppended(final String text,
						IStreamMonitor monitor) {

					if (text.indexOf("Address already in use") > -1) {  // *** Put the line that identifies the server is started (like WEBrick::HTTPServer#start
						// Notify the view
						stop();
					}
					
					// Log the output to the console
					consoleStream.print(text);
				}
			});
		} catch (CoreException e) {
			ServerLog.logError("Error running the server command", e);
			syncUpdateStatus(IServerConstants.STOPPED);
		}
	}

	public void stop() {
		if (serverProcess != null) {
			Thread stopThread = new Thread() {
				public void run() {
					syncUpdateStatus(IServerConstants.STOPPING);

						// Forcefully kill the server process
						while (!serverProcess.isTerminated()) {
							try {
								serverProcess.terminate();
								Thread.sleep(10);
							} catch (DebugException e) {
								// TODO Auto-generated catch block
								e.printStackTrace();
							} catch (InterruptedException e) {
								// TODO Auto-generated catch block
								e.printStackTrace();
							}
						}

					// Notify the view that the server has stopped
					syncUpdateStatus(IServerConstants.STOPPED);
				}
			};

			// Initiate trying to stop the server.
			stopThread.start();
		}
	}
	
	public void restart(){
		stop();
		start();
	}

}
