/*******************************************************************************
 * Copyright (c) 2005 RadRails.org and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Common Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.radrails.org/legal/cpl-v10.html
 *******************************************************************************/
package org.radrails.server.core.mongrel;

import java.io.File;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Platform;
import org.eclipse.core.runtime.Status;
import org.eclipse.core.runtime.jobs.Job;
import org.eclipse.debug.core.DebugException;
import org.eclipse.debug.core.DebugPlugin;
import org.eclipse.debug.core.ILaunch;
import org.eclipse.debug.core.ILaunchManager;
import org.eclipse.debug.core.IStreamListener;
import org.eclipse.debug.core.Launch;
import org.eclipse.debug.core.model.IProcess;
import org.eclipse.debug.core.model.IStreamMonitor;
import org.eclipse.swt.SWT;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.console.ConsolePlugin;
import org.eclipse.ui.console.IConsole;
import org.eclipse.ui.console.MessageConsoleStream;
import org.radrails.db.core.IDatabaseConstants;
import org.radrails.rails.internal.core.RailsPlugin;
import org.radrails.rails.internal.core.RailsRuntime;
import org.radrails.server.core.IServerConstants;
import org.radrails.server.core.Server;
import org.radrails.server.core.ServerConsole;
import org.radrails.server.core.ServerLog;
import org.radrails.server.core.ServerManager;
import org.radrails.server.core.ServerPlugin;

public class MongrelServer extends Server {

	private IProcess serverProcess;

	private MessageConsoleStream stdOutStream;
	
	private MessageConsoleStream stdErrStream;
	
	private boolean keepStarting;
	
	/**
	 * Constructor.
	 * 
	 * @param project
	 *            the location of the project the server is associated with
	 * @param name
	 *            the name of the server
	 * @param portNumber
	 *            the number of the port the server runs on
	 */
	public MongrelServer(File project, String name, String port,
			String environment) {
		super(project, name, IServerConstants.TYPE_MONGREL, port, environment);
	}

	/**
	 * Constructor. Environment defaults to development.
	 * 
	 * @param project
	 *            the location of the project the server is associated with
	 * @param name
	 *            the name of the server
	 * @param port
	 *            the number of the port the server runs on
	 */
	public MongrelServer(File project, String name, String port) {
		this(project, name, port, IDatabaseConstants.ENV_DEVELOPMENT);
	}

	/**
	 * Constructor. Name defaults to projectNameServer, port defaults to next
	 * available, and environment defaults to development.
	 * 
	 * @param project
	 *            the location of the project the server is associated with
	 */
	public MongrelServer(File project) {
		this(project, project.getName() + "Server", ServerManager.getInstance()
				.getNextAvailablePort(), IDatabaseConstants.ENV_DEVELOPMENT);
	}

	/**
	 * Empty constructor for use by the loadServers() method in ServerManager.
	 */
	public MongrelServer() {
		this(null, null, null, null);
	}
	
	public void start() {
		Job j = new Job("Starting server") {
			protected IStatus run(IProgressMonitor monitor) {
				startServer();
				return Status.OK_STATUS;
			}
		};
		j.schedule();
	}
	
	private void startServer() {
		syncUpdateStatus(IServerConstants.STARTING);

		// Prepare the server commands
		ArrayList commandList = new ArrayList();
		String mongrelPath= RailsPlugin.getInstance().getMongrelPath();
		if(mongrelPath == null || mongrelPath.equals("")) {
			mongrelPath = "mongrel_rails";
		}
		commandList.add(mongrelPath);
		commandList.add("start");
		commandList.add("--port=" + port);
		commandList.add("--environment=" + environment);
		
		try {
			// on win32, need to run the script with ruby
			Process proc = null;
			if(Platform.getOS().equals(Platform.OS_WIN32)) {
				proc = RailsRuntime.rubyExec(
						(String[])commandList.toArray(new String[]{}), project, false);
			} else {
				proc = RailsRuntime.exec(
						(String[])commandList.toArray(new String[]{}), project, false);
			}
			ILaunch launch = new Launch(null, ILaunchManager.RUN_MODE, null);
			serverProcess = DebugPlugin.newProcess(launch, proc, "command");

			// Create a new console page for the server process
			SimpleDateFormat dateFormat = new SimpleDateFormat(
					"MMM d, yyyy h:mm:ss aa");
			String date = dateFormat.format(new Date());

			ServerConsole myConsole = new ServerConsole(this, project.getName()
					+ " - Mongrel Server - (" + date + ")", null);
			stdOutStream = myConsole.newMessageStream();
			stdErrStream = myConsole.newMessageStream();
			PlatformUI.getWorkbench().getDisplay().syncExec(new Runnable() {
				public void run() {
					stdOutStream.setColor(PlatformUI.getWorkbench()
							.getDisplay().getSystemColor(SWT.COLOR_BLUE));
					stdErrStream.setColor(PlatformUI.getWorkbench()
							.getDisplay().getSystemColor(SWT.COLOR_RED));
				}
			});
			ConsolePlugin.getDefault().getConsoleManager().addConsoles(
					new IConsole[] { myConsole });

			// Prepare the stream listeners
			IStreamMonitor stdOut = serverProcess.getStreamsProxy()
					.getOutputStreamMonitor();
			IStreamMonitor stdErr = serverProcess.getStreamsProxy()
					.getErrorStreamMonitor();

			// Write stdout to the console
			stdOut.addListener(new IStreamListener() {
				public void streamAppended(final String text,
						IStreamMonitor monitor) {
					// Log the output to the console
					stdOutStream.print(text);
				}
			});

			// Write stderr to the console
			stdErr.addListener(new IStreamListener() {
				public void streamAppended(final String text,
						IStreamMonitor monitor) {
					// Log the output to the console
					stdErrStream.print(text);
				}
			});
			
			boolean available = true;
			keepStarting = true;
			while (available = ServerPlugin.isPortAvailable(port) && keepStarting) {
				try {
					Thread.sleep(10);
				} catch (InterruptedException e) {
					ServerLog.logError("Error sleeping", e);
				}
			}
			if (!available && keepStarting) {
				syncUpdateStatus(IServerConstants.STARTED);
			}
		} catch (CoreException e) {
			ServerLog.logError("Error running the server command", e);
			syncUpdateStatus(IServerConstants.STOPPED);
		}
	}
	
	public void stop() {
		if (serverProcess != null) {
			// Make sure to stop the start job, just in case the user is
			// stopping the server while it's still starting
			keepStarting = false;
			
			Job j = new Job("Stopping server") {
				protected IStatus run(IProgressMonitor monitor) {
					stopServer();
					return Status.OK_STATUS;
				}
			};
			j.schedule();
		}
	}

	private void stopServer() {
		syncUpdateStatus(IServerConstants.STOPPING);

		if (Platform.getOS().equals(Platform.OS_WIN32)) {
			while (!serverProcess.isTerminated()) {
				try {
					serverProcess.terminate();
					Thread.sleep(10);
				} catch (DebugException e) {
					ServerLog.logError("Error terminating server process", e);
				} catch (InterruptedException e) {
					ServerLog.logError("Error sleeping", e);
				}
			}
		} else {
			try {
				while (!serverProcess.isTerminated()) {
					// just in case
					serverProcess.terminate();
					Thread.sleep(10);
				}
			} catch (CoreException e) {
				ServerLog.logError(
						"Error running server process kill command", e);
			} catch (InterruptedException e) {
				ServerLog.logError("Error sleeping", e);
			}
		}

		// Notify the view that the server has stopped
		syncUpdateStatus(IServerConstants.STOPPED);
	}
	
	public void restart(){
		Job j = new Job("Restarting server") {
			protected IStatus run(IProgressMonitor monitor) {
				keepStarting = false;
				stopServer();
				startServer();
				return Status.OK_STATUS;
			}
		};
		j.schedule();
	}

}
