package org.radrails.server.internal.ui.actions;

import java.io.IOException;
import java.net.InetAddress;
import java.net.InetSocketAddress;
import java.net.Socket;
import java.util.List;

import org.eclipse.jface.action.IAction;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.ui.IViewPart;
import org.radrails.server.core.Server;
import org.radrails.server.internal.ui.ServerUILog;

/**
 * Helper methods for the server action delegates.
 * 
 * @author mkent
 * 
 */
public class ServerActionHelper {

	/**
	 * Starts a server. First checks the database connection of the server's
	 * project, then asks the user what to do if the connection fails.
	 * 
	 * @param viewPart
	 *            the view part of the action delegate (used as a parent for the
	 *            dialog)
	 * @param server
	 *            the server to start
	 */
	protected static void startServer(IViewPart viewPart, Server server) {
		// Check if the server can bind to the port
		boolean portAvailable = checkPortAvailable(server.getPort());

		// If port is available, start the server
		// otherwise, inform the user
		if (portAvailable) {
			server.start();
		} else {
			MessageDialog
					.openError(viewPart.getViewSite().getShell(),
							"Port already in use",
							"The server could not be started. The port is already in use.");
		}
	}

	/**
	 * Checks if the port is available to be bound to for the server.
	 * 
	 * @param port
	 *            The port to attempt to connect on.
	 * 
	 * @return <code>true</code> if the port is available for use,
	 *         <code>false</code> otherwise.
	 */
	protected static boolean checkPortAvailable(String port) {
		boolean available = true;

		Socket socket = null;
		try {
			// Get the int value of the port
			int intPort = Integer.parseInt(port);

			// Attempt to connect to the localhost on that port
			// If an error occurs, we weren't able to connect, which
			// means the port is available.
			socket = new Socket();
			socket.connect(new InetSocketAddress(InetAddress
					.getByName("localhost"), intPort));
			if (socket.isConnected()) {
				// Connection established, which means the port
				// is in use
				available = false;
			}
		} catch (NumberFormatException e) {
			// The port couldn't be parsed
			available = false;
		} catch (IOException e) {
			// Could not connect, the port is available
			available = true;
		} finally {
			// Close the socket if necessary
			if (socket != null && !socket.isClosed()) {
				try {
					socket.close();
				} catch (IOException e) {
					// Log this, even though it shouldn't happen
					ServerUILog.logError("Unable to close server socket.", e);
				}
			}
		}

		return available;
	}

	/**
	 * Enables an action. The <code>enabledFor</code> parameter is a list of
	 * states from <code>IServerConstants</code> for which the given action is
	 * enabled. Otherwise, the action is disabled.
	 * 
	 * @param action
	 *            the action to enable
	 * @param selection
	 *            the selected server
	 * @param enabledFor
	 *            the states for which the action is enabled
	 */
	protected static void enableAction(IAction action, ISelection selection,
			List enabledFor) {
		if (selection != null) {
			Server server = (Server) ((IStructuredSelection) selection)
					.getFirstElement();
			if (server != null) {
				String status = server.getStatus();
				if (enabledFor.contains(status)) {
					action.setEnabled(true);
				} else {
					action.setEnabled(false);
				}
			}
		}
	}
}
